@@ -27,6 +27,27 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: config.h,v $
+ * Revision 1.28  2005/11/25 03:43:47  csoutheren
+ * Fixed function argument comments to be compatible with Doxygen
+ *
+ * Revision 1.27  2005/01/26 05:37:44  csoutheren
+ * Added ability to remove config file support
+ *
+ * Revision 1.26  2004/01/06 21:16:38  csoutheren
+ * Added "virtual" keyword to methods to allow descendant classes.
+ * Thanks to Cristian Bullokles for pointing out this issue
+ *
+ * Revision 1.25  2003/09/17 05:41:58  csoutheren
+ * Removed recursive includes
+ *
+ * Revision 1.24  2003/09/17 01:18:02  csoutheren
+ * Removed recursive include file system and removed all references
+ * to deprecated coooperative threading support
+ *
+ * Revision 1.23  2002/09/16 01:08:59  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
  * Revision 1.22  2001/05/22 12:49:32  robertj
  * Did some seriously wierd rewrite of platform headers to eliminate the
  *   stupid GNU compiler warning about braces not matching.
@@ -94,12 +115,17 @@
  */
 
 
+#ifndef _PCONFIG
 #define _PCONFIG
 
-#ifdef __GNUC__
+#ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
+#include "ptbuildopts.h"
+#ifdef P_CONFIG_FILE
+
+class PXConfig;
 
 /** A class representing a configuration for the application.
 There are four sources of configuration information. The system environment,
@@ -160,41 +186,41 @@ class PConfig : public PObject
        active PConfig object.
      */
     PConfig(
-      Source src = Application  /// Standard source for the configuration.
+      Source src = Application  ///< Standard source for the configuration.
     );
     /** Create a new configuration object. */
     PConfig(
-      Source src,               /// Standard source for the configuration.
-      const PString & appname   /// Name of application
+      Source src,               ///< Standard source for the configuration.
+      const PString & appname   ///< Name of application
     );
     /** Create a new configuration object. */
     PConfig(
-      Source src,               /// Standard source for the configuration.
-      const PString & appname,  /// Name of application
-      const PString & manuf     /// Manufacturer
+      Source src,               ///< Standard source for the configuration.
+      const PString & appname,  ///< Name of application
+      const PString & manuf     ///< Manufacturer
     );
     /** Create a new configuration object. */
     PConfig(
-      const PString & section,  /// Default section to search for variables.
-      Source src = Application  /// Standard source for the configuration.
+      const PString & section,  ///< Default section to search for variables.
+      Source src = Application  ///< Standard source for the configuration.
     );
     /** Create a new configuration object. */
     PConfig(
-      const PString & section,  /// Default section to search for variables.
-      Source src,               /// Standard source for the configuration.
-      const PString & appname   /// Name of application
+      const PString & section,  ///< Default section to search for variables.
+      Source src,               ///< Standard source for the configuration.
+      const PString & appname   ///< Name of application
     );
     /** Create a new configuration object. */
     PConfig(
-      const PString & section,  /// Default section to search for variables.
-      Source src,               /// Standard source for the configuration.
-      const PString & appname,  /// Name of application
-      const PString & manuf     /// Manufacturer
+      const PString & section,  ///< Default section to search for variables.
+      Source src,               ///< Standard source for the configuration.
+      const PString & appname,  ///< Name of application
+      const PString & manuf     ///< Manufacturer
     );
     /** Create a new configuration object. */
     PConfig(
-      const PFilePath & filename, /// Explicit name of the configuration file.
-      const PString & section     /// Default section to search for variables.
+      const PFilePath & filename, ///< Explicit name of the configuration file.
+      const PString & section     ///< Default section to search for variables.
     );
   //@}
 
@@ -207,8 +233,8 @@ class PConfig : public PObject
        Note when the #Environment# source is being used the default
        section may be set but it is ignored.
      */
-    void SetDefaultSection(
-      const PString & section  /// New default section name.
+    virtual void SetDefaultSection(
+      const PString & section  ///< New default section name.
     );
 
     /** Get the default section for variable operations. All functions that deal
@@ -220,7 +246,7 @@ class PConfig : public PObject
 
        @return default section name string.
      */
-    PString GetDefaultSection() const;
+    virtual PString GetDefaultSection() const;
 
     /** Get all of the section names currently specified in the file. A section
        is the part specified by the [ and ] characters.
@@ -230,17 +256,17 @@ class PConfig : public PObject
 
        @return list of all section names.
      */
-    PStringList GetSections() const;
+    virtual PStringList GetSections() const;
 
     /** Get a list of all the keys in the section. If the section name is not
        specified then use the default section.
 
        @return list of all key names.
      */
-    PStringList GetKeys() const;
+    virtual PStringList GetKeys() const;
     /** Get a list of all the keys in the section. */
-    PStringList GetKeys(
-      const PString & section   /// Section to use instead of the default.
+    virtual PStringList GetKeys(
+      const PString & section   ///< Section to use instead of the default.
     ) const;
 
     /** Get all of the keys in the section and their values. If the section
@@ -248,10 +274,10 @@ class PConfig : public PObject
 
        @return Dictionary of all key names and their values.
      */
-    PStringToString GetAllKeyValues() const;
+    virtual PStringToString GetAllKeyValues() const;
     /** Get all of the keys in the section and their values. */
-    PStringToString GetAllKeyValues(
-      const PString & section   /// Section to use instead of the default.
+    virtual PStringToString GetAllKeyValues(
+      const PString & section   ///< Section to use instead of the default.
     ) const;
 
 
@@ -261,10 +287,10 @@ class PConfig : public PObject
        Note that the section header is also removed so the section will not
        appear in the GetSections() function.
      */
-    void DeleteSection();
+    virtual void DeleteSection();
     /** Delete all variables in the specified section. */
-    void DeleteSection(
-      const PString & section   /// Name of section to delete.
+    virtual void DeleteSection(
+      const PString & section   ///< Name of section to delete.
     );
 
     /** Delete the particular variable in the specified section. If the section
@@ -275,13 +301,13 @@ class PConfig : public PObject
        value without deleting the key, use SetString() to set it to the empty
        string.
      */
-    void DeleteKey(
-      const PString & key       /// Key of the variable to delete.
+    virtual void DeleteKey(
+      const PString & key       ///< Key of the variable to delete.
     );
     /** Delete the particular variable in the specified section. */
-    void DeleteKey(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key       /// Key of the variable to delete.
+    virtual void DeleteKey(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key       ///< Key of the variable to delete.
     );
 
     /**Determine if the particular variable in the section is actually present.
@@ -292,13 +318,13 @@ class PConfig : public PObject
        the default "DefVal" being used, or the user had explicitly saved the
        value "DefVal" into the PConfig.
      */
-    BOOL HasKey(
-      const PString & key       /// Key of the variable.
+    virtual BOOL HasKey(
+      const PString & key       ///< Key of the variable.
     ) const;
     /**Determine if the particular variable in the section is actually present. */
-    BOOL HasKey(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key       /// Key of the variable.
+    virtual BOOL HasKey(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key       ///< Key of the variable.
     ) const;
   //@}
 
@@ -314,33 +340,33 @@ class PConfig : public PObject
 
        @return string value of the variable.
      */
-    PString GetString(
-      const PString & key       /// The key name for the variable.
+    virtual PString GetString(
+      const PString & key       ///< The key name for the variable.
     ) const;
     /** Get a string variable determined by the key in the section. */
-    PString GetString(
-      const PString & key,      /// The key name for the variable.
-      const PString & dflt      /// Default value for the variable.
+    virtual PString GetString(
+      const PString & key,      ///< The key name for the variable.
+      const PString & dflt      ///< Default value for the variable.
     ) const;
     /** Get a string variable determined by the key in the section. */
-    PString GetString(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      const PString & dflt      /// Default value for the variable.
+    virtual PString GetString(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      const PString & dflt      ///< Default value for the variable.
     ) const;
 
     /** Set a string variable determined by the key in the section. If the
        section name is not specified then the default section is used.
      */
-    void SetString(
-      const PString & key,      /// The key name for the variable.
-      const PString & value     /// New value to set for the variable.
+    virtual void SetString(
+      const PString & key,      ///< The key name for the variable.
+      const PString & value     ///< New value to set for the variable.
     );
     /** Set a string variable determined by the key in the section. */
-    void SetString(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      const PString & value     /// New value to set for the variable.
+    virtual void SetString(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      const PString & value     ///< New value to set for the variable.
     );
 
 
@@ -360,15 +386,15 @@ class PConfig : public PObject
 
        @return boolean value of the variable.
      */
-    BOOL GetBoolean(
-      const PString & key,      /// The key name for the variable.
-      BOOL dflt = FALSE         /// Default value for the variable.
+    virtual BOOL GetBoolean(
+      const PString & key,      ///< The key name for the variable.
+      BOOL dflt = FALSE         ///< Default value for the variable.
     ) const;
     /** Get a boolean variable determined by the key in the section. */
-    BOOL GetBoolean(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      BOOL dflt = FALSE         /// Default value for the variable.
+    virtual BOOL GetBoolean(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      BOOL dflt = FALSE         ///< Default value for the variable.
     ) const;
 
     /** Set a boolean variable determined by the key in the section. If the
@@ -377,15 +403,15 @@ class PConfig : public PObject
        If value is TRUE then the string "True" is written to the variable
        otherwise the string "False" is set.
      */
-    void SetBoolean(
-      const PString & key,      /// The key name for the variable.
-      BOOL value                /// New value to set for the variable.
+    virtual void SetBoolean(
+      const PString & key,      ///< The key name for the variable.
+      BOOL value                ///< New value to set for the variable.
     );
     /** Set a boolean variable determined by the key in the section. */
-    void SetBoolean(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      BOOL value                /// New value to set for the variable.
+    virtual void SetBoolean(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      BOOL value                ///< New value to set for the variable.
     );
 
 
@@ -398,15 +424,15 @@ class PConfig : public PObject
 
        @return integer value of the variable.
      */
-    long GetInteger(
-      const PString & key,      /// The key name for the variable.
-      long dflt = 0             /// Default value for the variable.
+    virtual long GetInteger(
+      const PString & key,      ///< The key name for the variable.
+      long dflt = 0             ///< Default value for the variable.
     ) const;
     /* Get an integer variable determined by the key in the section. */
-    long GetInteger(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      long dflt = 0             /// Default value for the variable.
+    virtual long GetInteger(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      long dflt = 0             ///< Default value for the variable.
     ) const;
 
     /** Set an integer variable determined by the key in the section. If the
@@ -415,15 +441,15 @@ class PConfig : public PObject
        The value is always formatted as a signed number with no leading or
        trailing blanks.
      */
-    void SetInteger(
-      const PString & key,      /// The key name for the variable.
-      long value                /// New value to set for the variable.
+    virtual void SetInteger(
+      const PString & key,      ///< The key name for the variable.
+      long value                ///< New value to set for the variable.
     );
     /** Set an integer variable determined by the key in the section. */
-    void SetInteger(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      long value                /// New value to set for the variable.
+    virtual void SetInteger(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      long value                ///< New value to set for the variable.
     );
 
 
@@ -436,15 +462,15 @@ class PConfig : public PObject
 
        @return integer value of the variable.
      */
-    PInt64 GetInt64(
-      const PString & key,      /// The key name for the variable.
-      PInt64 dflt = 0           /// Default value for the variable.
+    virtual PInt64 GetInt64(
+      const PString & key,      ///< The key name for the variable.
+      PInt64 dflt = 0           ///< Default value for the variable.
     ) const;
     /** Get a 64 bit integer variable determined by the key in the section. */
-    PInt64 GetInt64(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      PInt64 dflt = 0           /// Default value for the variable.
+    virtual PInt64 GetInt64(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      PInt64 dflt = 0           ///< Default value for the variable.
     ) const;
 
     /** Set a 64 bit integer variable determined by the key in the section. If the
@@ -453,15 +479,15 @@ class PConfig : public PObject
        The value is always formatted as a signed number with no leading or
        trailing blanks.
      */
-    void SetInt64(
-      const PString & key,      /// The key name for the variable.
-      PInt64 value              /// New value to set for the variable.
+    virtual void SetInt64(
+      const PString & key,      ///< The key name for the variable.
+      PInt64 value              ///< New value to set for the variable.
     );
     /** Set a 64 bit integer variable determined by the key in the section. */
-    void SetInt64(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      PInt64 value              /// New value to set for the variable.
+    virtual void SetInt64(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      PInt64 value              ///< New value to set for the variable.
     );
 
 
@@ -474,15 +500,15 @@ class PConfig : public PObject
 
        @return floating point value of the variable.
      */
-    double GetReal(
-      const PString & key,      /// The key name for the variable.
-      double dflt = 0           /// Default value for the variable.
+    virtual double GetReal(
+      const PString & key,      ///< The key name for the variable.
+      double dflt = 0           ///< Default value for the variable.
     ) const;
     /** Get a floating point variable determined by the key in the section. */
-    double GetReal(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      double dflt = 0           /// Default value for the variable.
+    virtual double GetReal(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      double dflt = 0           ///< Default value for the variable.
     ) const;
 
     /** Set a floating point variable determined by the key in the section. If
@@ -492,15 +518,15 @@ class PConfig : public PObject
        number with no leading or trailing blanks, ie it uses the %g formatter
        from the printf() function.
      */
-    void SetReal(
-      const PString & key,      /// The key name for the variable.
-      double value              /// New value to set for the variable.
+    virtual void SetReal(
+      const PString & key,      ///< The key name for the variable.
+      double value              ///< New value to set for the variable.
     );
     /** Set a floating point variable determined by the key in the section. */
-    void SetReal(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      double value              /// New value to set for the variable.
+    virtual void SetReal(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      double value              ///< New value to set for the variable.
     );
 
     /** Get a #PTime# variable determined by the key in the section. If
@@ -512,38 +538,38 @@ class PConfig : public PObject
 
        @return time/date value of the variable.
      */
-    PTime GetTime(
-      const PString & key       /// The key name for the variable.
+    virtual PTime GetTime(
+      const PString & key       ///< The key name for the variable.
     ) const;
     /** Get a #PTime# variable determined by the key in the section. */
-    PTime GetTime(
-      const PString & key,      /// The key name for the variable.
-      const PTime & dflt        /// Default value for the variable.
+    virtual PTime GetTime(
+      const PString & key,      ///< The key name for the variable.
+      const PTime & dflt        ///< Default value for the variable.
     ) const;
     /** Get a #PTime# variable determined by the key in the section. */
-    PTime GetTime(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key       /// The key name for the variable.
+    virtual PTime GetTime(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key       ///< The key name for the variable.
     ) const;
     /** Get a #PTime# variable determined by the key in the section. */
-    PTime GetTime(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      const PTime & dflt        /// Default value for the variable.
+    virtual PTime GetTime(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      const PTime & dflt        ///< Default value for the variable.
     ) const;
 
     /** Set a #PTime# variable determined by the key in the section. If
        the section name is not specified then the default section is used.
      */
-    void SetTime(
-      const PString & key,      /// The key name for the variable.
-      const PTime & value       /// New value to set for the variable.
+    virtual void SetTime(
+      const PString & key,      ///< The key name for the variable.
+      const PTime & value       ///< New value to set for the variable.
     );
     /** Set a #PTime# variable determined by the key in the section. */
-    void SetTime(
-      const PString & section,  /// Section to use instead of the default.
-      const PString & key,      /// The key name for the variable.
-      const PTime & value       /// New value to set for the variable.
+    virtual void SetTime(
+      const PString & section,  ///< Section to use instead of the default.
+      const PString & key,      ///< The key name for the variable.
+      const PTime & value       ///< New value to set for the variable.
     );
   //@}
 
@@ -557,18 +583,25 @@ class PConfig : public PObject
   private:
     // Do common construction code.
     void Construct(
-      Source src,               /// Standard source for the configuration.
-      const PString & appname,  /// Name of application
-      const PString & manuf     /// Manufacturer
+      Source src,               ///< Standard source for the configuration.
+      const PString & appname,  ///< Name of application
+      const PString & manuf     ///< Manufacturer
     );
     void Construct(
-      const PFilePath & filename  /// Explicit name of the configuration file.
+      const PFilePath & filename  ///< Explicit name of the configuration file.
     );
 
 
 // Include platform dependent part of class
-#include <ptlib/config.h>
+#ifdef _WIN32
+#include "msos/ptlib/config.h"
+#else
+#include "unix/ptlib/config.h"
+#endif
 };
 
+#endif // P_CONFIG_FILE
+
+#endif
 
 // End Of File ///////////////////////////////////////////////////////////////
