@@ -25,6 +25,21 @@
  *                 Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
  * $Log: vidinput_v4l.cxx,v $
+ * Revision 1.15.2.9  2007/09/18 09:25:46  dsandras
+ * Killed GCC 4.2 warnings.
+ *
+ * Revision 1.15.2.8  2007/06/02 12:10:03  dsandras
+ * Added patch from Michael Smith <msmith cbnco com> refreshing video
+ * capabilities when appropriate. Thanks!
+ *
+ * Revision 1.15.2.7  2007/05/01 20:14:48  dsandras
+ * Fixed possible crash when opening V4L devices with 352x288 fixed
+ * width capability thanks Luc Saillard. (Ekiga #434223).
+ *
+ * Revision 1.15.2.6  2007/04/10 21:17:56  dsandras
+ * Added MJPEG support. Added workarounds for broken qspca driver.
+ * Thanks to Luc Saillard (luc saillard org).
+ *
  * Revision 1.15.2.5  2007/01/03 22:37:37  dsandras
  * Backports from HEAD.
  *
@@ -276,23 +291,33 @@ PCREATE_VIDINPUT_PLUGIN(V4L);
 #define HINT_FORCE_DBLBUF                   0x0400  /// Force double buffering on quickcam express
 
 static struct {
-  char     *name_regexp;        // String used to match the driver name
-  char     *name;               // String used for ptrace output
-  char     *version;             // Apply the hint if kernel
+  const char     *name_regexp;        // String used to match the driver name
+  const char     *name;               // String used for ptrace output
+  const char     *version;             // Apply the hint if kernel
                                 // version < given version,
                                 // 0 means always apply
   unsigned hints;               // Hint flags
   int      pref_palette;        // Preferred palette.
 } driver_hints[] = {
 
-    /**Philips usb web cameras
+    /**Most of usb web cameras from the spca5xx driver
+     */
+    
+  { "^Broken sensor chipset that accept only 640x480$",
+    "Broken sensor chipset that accept only 640x480",
+    NULL,
+    HINT_ALWAYS_WORKS_640_480|
+    HINT_CGWIN_FAILS,
+    0},
+
+     /**Philips usb web cameras
        Native format is 420(P) so use it.
      */
     
   { "^Philips [0-9]+ webcam$",
     "Philips USB webcam",
     NULL,
-    HINT_HAS_PREF_PALETTE,
+    HINT_ALWAYS_WORKS_640_480,
     VIDEO_PALETTE_YUV420P },
   
   /**Brooktree based capture boards.
@@ -386,6 +411,53 @@ static struct {
 
 };
 
+/*
+ * This is list of channel names that accept only fixed size resolution
+ * The spca5xx driver store in the channel name, the name of the bridge,
+ * not the sensor, so we use a second list to trim false positive.
+ */
+static const char *bridges_with_640x480_fixed_width[] = {
+   "SPCA505",
+   "SPCA506",
+   "SPCA501",
+   "SPCA504",
+   "SPCA500", /* Only the LogitechClickSmart310 doesn't support the 640x480 */
+   "SPCA504B",
+   "SPCA504C",
+   "SPCA536",
+   "SN9C102", /* SENSOR_PAS106 and SENSOR_TAS5110 doesn't support the 640x480 */
+   "ZC301-2", /* SENSOR_PAS106 doesn't support the 640x480 */
+   "CX11646",
+   "SN9CXXX",
+   "MR97311",
+   "VC0321",
+};
+
+static const char *sensors_with_352x288_fixed_width[] = {
+   "Philips SPC200NC ",	/* Using the SENSOR_PAS106 sensor */
+   "Philips SPC210NC (FB) ",
+   "Philips SPC300NC ",
+   "Creative NX",
+   "Creative Instant P0620",
+   "Creative Instant P0620D",
+   "Sonix sn9c10x + Pas106 sensor",
+   "Genius VideoCAM NB", /* Using the SENSOR_TAS5110 sensor */
+   "Sweex SIF webcam",
+   "Logitech ClickSmart 310", /* Using the SENSOR_HDCS1020 sensor */
+};
+
+#if 0
+/* TODO: We need to do the same hack */
+static const char *bridges_with_352x288_fixed_width[] = {
+   "SPCA508", /* 352x288 */
+   "SPCA561", /* 352x288 */
+   "TV8532", /* 352x288 */
+   "ET61XX51", /* 320x240 */
+   //  "SPCA533", /* 464*480 */
+   //  "PAC207BCA", /* 320x240 only */
+};
+#endif
+
 #define HINT(h) ((driver_hints[hint_index].hints & h) ? TRUE : FALSE)
 #define MAJOR(a) (int)((unsigned short) (a) >> 8)
 #define MINOR(a) (int)((unsigned short) (a) & 0xFF) 
@@ -631,6 +703,13 @@ PVideoInputDevice_V4L::~PVideoInputDevice_V4L()
 {
     Close();
 }
+
+/* From the spca5xx driver and gspca driver */
+#ifndef VIDEO_PALETTE_RAW_JPEG
+#define VIDEO_PALETTE_RAW_JPEG  20
+#define VIDEO_PALETTE_JPEG 21
+#endif
+
 static struct {
   const char * colourFormat;
   int code;
@@ -647,7 +726,8 @@ static struct {
   { "YUV420", VIDEO_PALETTE_YUV420 },
   { "YUV420P", VIDEO_PALETTE_YUV420P },
   { "YUV410P", VIDEO_PALETTE_YUV410P },
-  { "UYVY422", VIDEO_PALETTE_UYVY }
+  { "UYVY422", VIDEO_PALETTE_UYVY },
+  { "MJPEG", VIDEO_PALETTE_JPEG }
 };
 
 
@@ -675,8 +755,7 @@ BOOL PVideoInputDevice_V4L::Open(const PString & devName, BOOL startImmediate)
   }
   
   // get the device capabilities
-  if (::ioctl(videoFd, VIDIOCGCAP, &videoCapability) < 0)  {
-    PTRACE(1,"PVideoInputDevice_V4L:: get device capablilities failed : "<< ::strerror(errno));
+  if (!RefreshCapabilities()) {
     ::close (videoFd);
     videoFd = -1;
     return FALSE;
@@ -720,7 +799,41 @@ BOOL PVideoInputDevice_V4L::Open(const PString & devName, BOOL startImmediate)
     }
   }
 
-
+  /*
+   * Some drivers like the spca5xx ou the gspca returns OK for any resolution
+   * between min-max. But the image is crop, and the user doesn't see his face
+   * entirely.
+   * The problem is the drive support more than 200 webcams, and we cannot add
+   * them all to the list of driver_hints[], so we use the sensor description
+   * to enable the hack. The channel name contains the name of the bridge, so
+   * we have only 10 comp to be. But some webcams have a 640x480 bridge, and a
+   * small sensor ...
+   */
+  if (hint_index >= PARRAYSIZE(driver_hints)-1) {
+     struct video_channel channel;
+     memset(&channel, 0, sizeof(struct video_channel));
+     if (::ioctl(videoFd, VIDIOCGCHAN, &channel) == 0) {
+	/* Only check if the called doesn't return an error */
+	for (tbl = 0; tbl < PARRAYSIZE(bridges_with_640x480_fixed_width); tbl ++) {
+	   if (strcmp(bridges_with_640x480_fixed_width[tbl], channel.name) == 0) {
+	      BOOL false_positive = FALSE;
+	      unsigned int idx;
+	      for (idx = 0; idx < PARRAYSIZE(sensors_with_352x288_fixed_width); idx++) {
+		 if (strcmp(sensors_with_352x288_fixed_width[idx], videoCapability.name) == 0) {
+		    false_positive = TRUE;
+		    break;
+		 }
+	      }
+	      if (false_positive == FALSE) {
+		 PTRACE(1,"PVideoInputDevice_V4L::Open: Found fixed 640x480 sensor");
+		 hint_index = 0;
+		 break;
+	      }
+	   }
+	}
+     }
+  }
+ 
   // Force double-buffering with buggy Quickcam driver.
   if (HINT (HINT_FORCE_DBLBUF)) {
 
@@ -834,8 +947,11 @@ BOOL PVideoInputDevice_V4L::SetVideoFormat(VideoFormat newFormat)
   channel.norm = fmt[newFormat];
 
   // set the information
-  if (::ioctl(videoFd, VIDIOCSCHAN, &channel) >= 0)
+  if (::ioctl(videoFd, VIDIOCSCHAN, &channel) >= 0) {
+    // format change might affect frame size limits; grab them again
+    RefreshCapabilities();
     return TRUE;
+  }
 
   PTRACE(1,"VideoInputDevice SetChannel failed : "<< ::strerror(errno));  
 
@@ -885,6 +1001,9 @@ BOOL PVideoInputDevice_V4L::SetChannel(int newChannel)
     return FALSE;
   }
   
+  // it's unlikely that channel change would affect frame size limits,
+  // but grab them again all the same
+  RefreshCapabilities();
   return TRUE;
 }
 
@@ -923,6 +1042,8 @@ BOOL PVideoInputDevice_V4L::SetVideoChannelFormat (int newNumber, VideoFormat vi
     return FALSE;
   }
 
+  // format change may affect frame size limits
+  RefreshCapabilities();
   return TRUE;
 }
 
@@ -1509,4 +1630,14 @@ BOOL PVideoInputDevice_V4L::TestAllFormats()
   return TRUE;
 }
 
+BOOL PVideoInputDevice_V4L::RefreshCapabilities()
+{
+  if (::ioctl(videoFd, VIDIOCGCAP, &videoCapability) < 0)  {
+    PTRACE(1,"PVideoInputV4lDevice:: get device capablilities failed : "<< ::strerror(errno));
+    return FALSE;
+  }
+  return TRUE;
+}
+
+
 // End Of File ///////////////////////////////////////////////////////////////
