@@ -5,7 +5,7 @@
  *
  * Portable Windows Library
  *
- * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
+ * Copyright (c) 1993-2002 Equivalence Pty. Ltd.
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -21,12 +21,65 @@
  *
  * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
  *
- * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
- * All Rights Reserved.
- *
  * Contributor(s): ______________________________________.
  *
  * $Log: cypher.cxx,v $
+ * Revision 1.47  2005/01/26 05:37:57  csoutheren
+ * Added ability to remove config file support
+ *
+ * Revision 1.46  2004/07/06 10:12:52  csoutheren
+ * Added static integer o factory template to assist in ensuring factories are instantiated
+ *
+ * Revision 1.45  2004/06/29 12:22:22  rjongbloed
+ * Fixed incorrect usage of result (now object rather than scalar), thanks Michal Zygmuntowicz
+ *
+ * Revision 1.44  2004/04/09 06:52:17  rjongbloed
+ * Removed #pargma linker command for /delayload of DLL as documentations sais that
+ *   you cannot do this.
+ *
+ * Revision 1.43  2004/04/03 23:53:09  csoutheren
+ * Added various changes to improce compatibility with the Sun Forte compiler
+ *   Thanks to Brian Cameron
+ * Added detection of readdir_r version
+ *
+ * Revision 1.42  2004/03/23 05:59:17  csoutheren
+ * Moved the Base64 routines into cypher.cxx, which is a more sensible
+ * place and reduces the inclusion of unrelated code
+ *
+ * Revision 1.41  2004/03/14 10:03:47  rjongbloed
+ * Fixed "security patch" that cleared entire object (including the vtable!) isntead of
+ *   clearing the "sensitive" information it was supposed to clear.
+ *
+ * Revision 1.40  2004/03/02 12:08:27  rjongbloed
+ * Added missing pragmas to automatically include libraries for OpenSSL
+ *
+ * Revision 1.39  2004/02/23 23:52:19  csoutheren
+ * Added pragmas to avoid every Windows application needing to include libs explicitly
+ *
+ * Revision 1.38  2003/04/27 23:52:57  craigs
+ * Fixed problem with SHA1 not calling Start
+ *
+ * Revision 1.37  2003/04/17 12:12:59  robertj
+ * Added windows library inclusion for optional openssl.
+ *
+ * Revision 1.36  2003/04/17 07:34:46  robertj
+ * Fixed correct test for P_SSL
+ *
+ * Revision 1.35  2003/04/17 01:21:55  craigs
+ * Fixed problem with delete'ing a void *
+ *
+ * Revision 1.33  2003/04/10 07:14:27  craigs
+ * Fixed link problem in MD5 class
+ *
+ * Revision 1.32  2003/04/10 06:16:09  craigs
+ * Added SHA-1 digest
+ *
+ * Revision 1.31  2002/11/06 22:47:24  robertj
+ * Fixed header comment (copyright etc)
+ *
+ * Revision 1.30  2002/06/05 12:29:15  craigs
+ * Changes for gcc 3.1
+ *
  * Revision 1.29  2001/03/01 03:55:59  robertj
  * Fixed MSVC warnings.
  *
@@ -127,12 +180,331 @@
 #pragma implementation "cypher.h"
 #endif
 
+#define P_DISABLE_FACTORY_INSTANCES
+
 #include <ptlib.h>
 #include <ptclib/cypher.h>
 #include <ptclib/mime.h>
 #include <ptclib/random.h>
 
 
+
+///////////////////////////////////////////////////////////////////////////////
+// PBase64
+
+PBase64::PBase64()
+{
+  StartEncoding();
+  StartDecoding();
+}
+
+
+void PBase64::StartEncoding(BOOL useCRLF)
+{
+  encodedString = "";
+  encodeLength = nextLine = saveCount = 0;
+  useCRLFs = useCRLF;
+}
+
+
+void PBase64::ProcessEncoding(const PString & str)
+{
+  ProcessEncoding((const char *)str);
+}
+
+
+void PBase64::ProcessEncoding(const char * cstr)
+{
+  ProcessEncoding((const BYTE *)cstr, strlen(cstr));
+}
+
+
+void PBase64::ProcessEncoding(const PBYTEArray & data)
+{
+  ProcessEncoding(data, data.GetSize());
+}
+
+
+static const char Binary2Base64[65] =
+            "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+void PBase64::OutputBase64(const BYTE * data)
+{
+  char * out = encodedString.GetPointer(((encodeLength+7)&~255) + 256);
+
+  out[encodeLength++] = Binary2Base64[data[0] >> 2];
+  out[encodeLength++] = Binary2Base64[((data[0]&3)<<4) | (data[1]>>4)];
+  out[encodeLength++] = Binary2Base64[((data[1]&15)<<2) | (data[2]>>6)];
+  out[encodeLength++] = Binary2Base64[data[2]&0x3f];
+
+  if (++nextLine > 18) { // 76 columns
+    if (useCRLFs)
+      out[encodeLength++] = '\r';
+    out[encodeLength++] = '\n';
+    nextLine = 0;
+  }
+}
+
+
+void PBase64::ProcessEncoding(const void * dataPtr, PINDEX length)
+{
+  if (length == 0)
+    return;
+
+  const BYTE * data = (const BYTE *)dataPtr;
+  while (saveCount < 3) {
+    saveTriple[saveCount++] = *data++;
+    if (--length == 0)
+      return;
+  }
+
+  OutputBase64(saveTriple);
+
+  PINDEX i;
+  for (i = 0; i+2 < length; i += 3)
+    OutputBase64(data+i);
+
+  saveCount = length - i;
+  switch (saveCount) {
+    case 2 :
+      saveTriple[0] = data[i++];
+      saveTriple[1] = data[i];
+      break;
+    case 1 :
+      saveTriple[0] = data[i];
+  }
+}
+
+
+PString PBase64::GetEncodedString()
+{
+  PString retval = encodedString;
+  encodedString = "";
+  encodeLength = 0;
+  return retval;
+}
+
+
+PString PBase64::CompleteEncoding()
+{
+  char * out = encodedString.GetPointer(encodeLength + 5)+encodeLength;
+
+  switch (saveCount) {
+    case 1 :
+      *out++ = Binary2Base64[saveTriple[0] >> 2];
+      *out++ = Binary2Base64[(saveTriple[0]&3)<<4];
+      *out++ = '=';
+      *out   = '=';
+      break;
+
+    case 2 :
+      *out++ = Binary2Base64[saveTriple[0] >> 2];
+      *out++ = Binary2Base64[((saveTriple[0]&3)<<4) | (saveTriple[1]>>4)];
+      *out++ = Binary2Base64[((saveTriple[1]&15)<<2)];
+      *out   = '=';
+  }
+
+  return encodedString;
+}
+
+
+PString PBase64::Encode(const PString & str)
+{
+  return Encode((const char *)str);
+}
+
+
+PString PBase64::Encode(const char * cstr)
+{
+  return Encode((const BYTE *)cstr, strlen(cstr));
+}
+
+
+PString PBase64::Encode(const PBYTEArray & data)
+{
+  return Encode(data, data.GetSize());
+}
+
+
+PString PBase64::Encode(const void * data, PINDEX length)
+{
+  PBase64 encoder;
+  encoder.ProcessEncoding(data, length);
+  return encoder.CompleteEncoding();
+}
+
+
+void PBase64::StartDecoding()
+{
+  perfectDecode = TRUE;
+  quadPosition = 0;
+  decodedData.SetSize(0);
+  decodeSize = 0;
+}
+
+
+BOOL PBase64::ProcessDecoding(const PString & str)
+{
+  return ProcessDecoding((const char *)str);
+}
+
+
+BOOL PBase64::ProcessDecoding(const char * cstr)
+{
+  static const BYTE Base642Binary[256] = {
+    96, 99, 99, 99, 99, 99, 99, 99, 99, 99, 98, 99, 99, 98, 99, 99,
+    99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99,
+    99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 62, 99, 99, 99, 63,
+    52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 99, 99, 99, 97, 99, 99,
+    99,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
+    15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 99, 99, 99, 99, 99,
+    99, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
+    41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 99, 99, 99, 99, 99,
+    99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99,
+    99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99,
+    99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99,
+    99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99,
+    99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99,
+    99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99,
+    99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99,
+    99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99
+  };
+
+  for (;;) {
+    BYTE value = Base642Binary[(BYTE)*cstr++];
+    switch (value) {
+      case 96 : // end of string
+        return FALSE;
+
+      case 97 : // '=' sign
+        if (quadPosition == 3 || (quadPosition == 2 && *cstr == '=')) {
+          quadPosition = 0;  // Reset this to zero, as have a perfect decode
+          return TRUE; // Stop decoding now as must be at end of data
+        }
+        perfectDecode = FALSE;  // Ignore '=' sign but flag decode as suspect
+        break;
+
+      case 98 : // CRLFs
+        break;  // Ignore totally
+
+      case 99 :  // Illegal characters
+        perfectDecode = FALSE;  // Ignore rubbish but flag decode as suspect
+        break;
+
+      default : // legal value from 0 to 63
+        BYTE * out = decodedData.GetPointer(((decodeSize+1)&~255) + 256);
+        switch (quadPosition) {
+          case 0 :
+            out[decodeSize] = (BYTE)(value << 2);
+            break;
+          case 1 :
+            out[decodeSize++] |= (BYTE)(value >> 4);
+            out[decodeSize] = (BYTE)((value&15) << 4);
+            break;
+          case 2 :
+            out[decodeSize++] |= (BYTE)(value >> 2);
+            out[decodeSize] = (BYTE)((value&3) << 6);
+            break;
+          case 3 :
+            out[decodeSize++] |= (BYTE)value;
+            break;
+        }
+        quadPosition = (quadPosition+1)&3;
+    }
+  }
+}
+
+
+PBYTEArray PBase64::GetDecodedData()
+{
+  perfectDecode = quadPosition == 0;
+  decodedData.SetSize(decodeSize);
+  PBYTEArray retval = decodedData;
+  retval.MakeUnique();
+  decodedData.SetSize(0);
+  decodeSize = 0;
+  return retval;
+}
+
+
+BOOL PBase64::GetDecodedData(void * dataBlock, PINDEX length)
+{
+  perfectDecode = quadPosition == 0;
+  BOOL bigEnough = length >= decodeSize;
+  memcpy(dataBlock, decodedData, bigEnough ? decodeSize : length);
+  decodedData.SetSize(0);
+  decodeSize = 0;
+  return bigEnough;
+}
+
+
+PString PBase64::Decode(const PString & str)
+{
+  PBYTEArray data;
+  Decode(str, data);
+  return PString((const char *)(const BYTE *)data, data.GetSize());
+}
+
+
+BOOL PBase64::Decode(const PString & str, PBYTEArray & data)
+{
+  PBase64 decoder;
+  decoder.ProcessDecoding(str);
+  data = decoder.GetDecodedData();
+  return decoder.IsDecodeOK();
+}
+
+
+BOOL PBase64::Decode(const PString & str, void * dataBlock, PINDEX length)
+{
+  PBase64 decoder;
+  decoder.ProcessDecoding(str);
+  return decoder.GetDecodedData(dataBlock, length);
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+// PMessageDigest
+
+PMessageDigest::PMessageDigest()
+{
+}
+
+void PMessageDigest::Process(const PString & str)
+{
+  Process((const char *)str);
+}
+
+
+void PMessageDigest::Process(const char * cstr)
+{
+  Process(cstr, strlen(cstr));
+}
+
+
+void PMessageDigest::Process(const PBYTEArray & data)
+{
+  Process(data, data.GetSize());
+}
+
+void PMessageDigest::Process(const void * dataBlock, PINDEX length)
+{
+  InternalProcess(dataBlock, length);
+}
+
+PString PMessageDigest::CompleteDigest()
+{
+  Result result;
+  CompleteDigest(result);
+  return PBase64::Encode(result.GetPointer(), result.GetSize());
+}
+
+void PMessageDigest::CompleteDigest(Result & result)
+{
+  InternalCompleteDigest(result);
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // PMessageDigest5
 
@@ -296,26 +668,7 @@ void PMessageDigest5::Start()
   count = 0;
 }
 
-
-void PMessageDigest5::Process(const PString & str)
-{
-  Process((const char *)str);
-}
-
-
-void PMessageDigest5::Process(const char * cstr)
-{
-  Process(cstr, strlen(cstr));
-}
-
-
-void PMessageDigest5::Process(const PBYTEArray & data)
-{
-  Process(data, data.GetSize());
-}
-
-
-void PMessageDigest5::Process(const void * dataPtr, PINDEX length)
+void PMessageDigest5::InternalProcess(const void * dataPtr, PINDEX length)
 {
   const BYTE * data = (const BYTE *)dataPtr;
 
@@ -344,15 +697,7 @@ void PMessageDigest5::Process(const void * dataPtr, PINDEX length)
 }
 
 
-PString PMessageDigest5::Complete()
-{
-  Code result;
-  Complete(result);
-  return PBase64::Encode(&result, sizeof(result));
-}
-
-
-void PMessageDigest5::Complete(Code & result)
+void PMessageDigest5::InternalCompleteDigest(Result & result)
 {
   // Put the count into bytes platform independently
   PUInt64l countBytes = count;
@@ -371,11 +716,13 @@ void PMessageDigest5::Complete(Code & result)
   Process(&countBytes, sizeof(countBytes));
 
   // Store state in digest
+  PUInt32l * valuep = (PUInt32l *)result.value.GetPointer(4 * sizeof(PUInt32l));
   for (PINDEX i = 0; i < PARRAYSIZE(state); i++)
-    result.value[i] = state[i];
+    valuep[i] = state[i];
 
   // Zeroize sensitive information.
-  memset(this, 0, sizeof(*this));
+  memset(buffer, 0, sizeof(buffer));
+  memset(state, 0, sizeof(state));
 }
 
 
@@ -385,7 +732,7 @@ PString PMessageDigest5::Encode(const PString & str)
 }
 
 
-void PMessageDigest5::Encode(const PString & str, Code & result)
+void PMessageDigest5::Encode(const PString & str, Result & result)
 {
   Encode((const char *)str, result);
 }
@@ -397,7 +744,7 @@ PString PMessageDigest5::Encode(const char * cstr)
 }
 
 
-void PMessageDigest5::Encode(const char * cstr, Code & result)
+void PMessageDigest5::Encode(const char * cstr, Result & result)
 {
   Encode((const BYTE *)cstr, strlen(cstr), result);
 }
@@ -409,7 +756,7 @@ PString PMessageDigest5::Encode(const PBYTEArray & data)
 }
 
 
-void PMessageDigest5::Encode(const PBYTEArray & data, Code & result)
+void PMessageDigest5::Encode(const PBYTEArray & data, Result & result)
 {
   Encode(data, data.GetSize(), result);
 }
@@ -417,20 +764,162 @@ void PMessageDigest5::Encode(const PBYTEArray & data, Code & result)
 
 PString PMessageDigest5::Encode(const void * data, PINDEX length)
 {
-  Code result;
+  Result result;
   Encode(data, length, result);
-  return PBase64::Encode(&result, sizeof(result));
+  return PBase64::Encode(result.GetPointer(), result.GetSize());
 }
 
 
-void PMessageDigest5::Encode(const void * data, PINDEX len, Code & result)
+void PMessageDigest5::Encode(const void * data, PINDEX len, Result & result)
 {
   PMessageDigest5 stomach;
   stomach.Process(data, len);
-  stomach.Complete(result);
+  stomach.CompleteDigest(result);
+}
+
+////  backwards compatability functions
+
+void PMessageDigest5::Encode(const PString & str, Code & result)
+{
+  Encode((const char *)str, result);
+}
+
+void PMessageDigest5::Encode(const char * cstr, Code & result)
+{
+  Encode((const BYTE *)cstr, strlen(cstr), result);
+}
+
+void PMessageDigest5::Encode(const PBYTEArray & data, Code & result)
+{
+  Encode(data, data.GetSize(), result);
+}
+
+void PMessageDigest5::Encode(const void * data, PINDEX len, Code & codeResult)
+{
+  PMessageDigest5 stomach;
+  stomach.Process(data, len);
+  stomach.Complete(codeResult);
+}
+
+PString PMessageDigest5::Complete()
+{
+  Code result;
+  Complete(result);
+  return PBase64::Encode(&result, sizeof(result));
+}
+
+void PMessageDigest5::Complete(Code & codeResult)
+{
+  Result result;
+  InternalCompleteDigest(result);
+  memcpy(codeResult.value, result.GetPointer(), sizeof(codeResult.value));
+}
+
+///////////////////////////////////////////////////////////////////////////////
+// PMessageDigestSHA1
+
+#if P_SSL
+
+#include <openssl/sha.h>
+
+#ifdef _MSC_VER
+
+#pragma comment(lib, P_SSL_LIB1)
+#pragma comment(lib, P_SSL_LIB2)
+
+#endif
+
+
+PMessageDigestSHA1::PMessageDigestSHA1()
+{
+  shaContext = NULL;
+  Start();
+}
+
+PMessageDigestSHA1::~PMessageDigestSHA1()
+{
+  delete (SHA_CTX *)shaContext;
+}
+
+void PMessageDigestSHA1::Start()
+{
+  delete (SHA_CTX *)shaContext;
+  shaContext = new SHA_CTX;
+
+  SHA1_Init((SHA_CTX *)shaContext);
+}
+
+void PMessageDigestSHA1::InternalProcess(const void * data, PINDEX len)
+{
+  if (shaContext == NULL)
+    return;
+
+  SHA1_Update((SHA_CTX *)shaContext, data, (unsigned long)len);
+}
+
+void PMessageDigestSHA1::InternalCompleteDigest(Result & result)
+{
+  if (shaContext == NULL)
+    return;
+
+  SHA1_Final(result.value.GetPointer(20), (SHA_CTX *)shaContext);
+  delete ((SHA_CTX *)shaContext);
+  shaContext = NULL;
 }
 
 
+PString PMessageDigestSHA1::Encode(const PString & str)
+{
+  return Encode((const char *)str);
+}
+
+
+void PMessageDigestSHA1::Encode(const PString & str, Result & result)
+{
+  Encode((const char *)str, result);
+}
+
+
+PString PMessageDigestSHA1::Encode(const char * cstr)
+{
+  return Encode((const BYTE *)cstr, strlen(cstr));
+}
+
+
+void PMessageDigestSHA1::Encode(const char * cstr, Result & result)
+{
+  Encode((const BYTE *)cstr, strlen(cstr), result);
+}
+
+
+PString PMessageDigestSHA1::Encode(const PBYTEArray & data)
+{
+  return Encode(data, data.GetSize());
+}
+
+
+void PMessageDigestSHA1::Encode(const PBYTEArray & data, Result & result)
+{
+  Encode(data, data.GetSize(), result);
+}
+
+
+PString PMessageDigestSHA1::Encode(const void * data, PINDEX length)
+{
+  Result result;
+  Encode(data, length, result);
+  return PBase64::Encode(result.GetPointer(), result.GetSize());
+}
+
+
+void PMessageDigestSHA1::Encode(const void * data, PINDEX len, Result & result)
+{
+  PMessageDigestSHA1 stomach;
+  stomach.Process(data, len);
+  stomach.CompleteDigest(result);
+}
+
+#endif
 
 ///////////////////////////////////////////////////////////////////////////////
 // PCypher
@@ -626,7 +1115,7 @@ void PTEACypher::GetKey(Key & newKey) const
 void PTEACypher::GenerateKey(Key & newKey)
 {
   static PRandom rand; //=1 // Explicitly set seed if need known random sequence
-  for (PINDEX i = 0; i < sizeof(Key); i++)
+  for (size_t i = 0; i < sizeof(Key); i++)
     newKey.value[i] = (BYTE)rand;
 }
 
@@ -675,6 +1164,8 @@ void PTEACypher::DecodeBlock(const void * in, void * out)
 ///////////////////////////////////////////////////////////////////////////////
 // PSecureConfig
 
+#ifdef P_CONFIG_FILE
+
 static const char DefaultSecuredOptions[] = "Secured Options";
 static const char DefaultSecurityKey[] = "Validation";
 static const char DefaultExpiryDateKey[] = "Expiry Date";
@@ -684,7 +1175,7 @@ static const char DefaultPendingPrefix[] = "Pending:";
 PSecureConfig::PSecureConfig(const PTEACypher::Key & prodKey,
                              const PStringArray & secKeys,
                              Source src)
-  : PConfig(DefaultSecuredOptions, src),
+  : PConfig(PString(DefaultSecuredOptions), src),
     securedKeys(secKeys),
     securityKey(DefaultSecurityKey),
     expiryDateKey(DefaultExpiryDateKey),
@@ -699,7 +1190,7 @@ PSecureConfig::PSecureConfig(const PTEACypher::Key & prodKey,
                              const char * const * secKeys,
                              PINDEX count,
                              Source src)
-  : PConfig(DefaultSecuredOptions, src),
+  : PConfig(PString(DefaultSecuredOptions), src),
     securedKeys(count, secKeys),
     securityKey(DefaultSecurityKey),
     expiryDateKey(DefaultExpiryDateKey),
@@ -837,5 +1328,6 @@ void PSecureConfig::ResetPending()
   DeleteKey(optionBitsKey);
 }
 
+#endif // P_CONFIG_FILE
 
 ///////////////////////////////////////////////////////////////////////////////
