@@ -27,6 +27,83 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: collect.cxx,v $
+ * Revision 1.72  2005/11/30 12:47:41  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.71  2005/11/25 01:01:15  csoutheren
+ * Applied patch #1351168
+ * PWlib various fixes
+ *
+ * Revision 1.70  2005/01/09 06:35:05  rjongbloed
+ * Fixed ability to make Clone() or MakeUnique() of a sorted list.
+ *
+ * Revision 1.69  2004/04/24 07:01:04  rjongbloed
+ * Fixed breaking of all lists with PAssertNULL chane. Oops.
+ *
+ * Revision 1.68  2004/04/24 06:27:56  rjongbloed
+ * Fixed GCC 3.4.0 warnings about PAssertNULL and improved recoverability on
+ *   NULL pointer usage in various bits of code.
+ *
+ * Revision 1.67  2004/04/03 08:22:21  csoutheren
+ * Remove pseudo-RTTI and replaced with real RTTI
+ *
+ * Revision 1.66  2004/03/23 10:59:35  rjongbloed
+ * Added some extra bulletproofing of containers to avoid complaints by some
+ *   memory checking tools, thanks Ted Szoczei
+ *
+ * Revision 1.65  2004/02/15 03:04:52  rjongbloed
+ * Fixed problem with PSortedList nil variable and assignment between instances,
+ *   pointed out by Ben Lear.
+ *
+ * Revision 1.64  2004/02/09 06:23:32  csoutheren
+ * Added fix for gcc 3.3.1 problem. Apparently, it is unable to correctly resolve
+ * a function argument that is a reference to a const pointer. Changing the argument
+ * to be a pointer to a pointer solves the problem. Go figure
+ *
+ * Revision 1.63  2004/02/08 22:46:35  csoutheren
+ * Added casts to fix problems with gcc
+ *
+ * Revision 1.62  2004/02/08 11:13:20  rjongbloed
+ * Fixed crash in heavily loaded multi-threaded systems using simultaneous sorted
+ *   lists, Thanks Federico Pinna, Fabrizio Ammollo and the gang at Reitek S.p.A.
+ *
+ * Revision 1.61  2003/09/24 22:13:58  dereksmithies
+ * Add fix for deleting an object, then readding it. Thanks Fabrizio Ammollo
+ *
+ * Revision 1.60  2003/08/31 22:11:30  dereksmithies
+ * Fix from Diego Tartara for the SetAt function. Many thanks.
+ *
+ * Revision 1.59  2002/12/02 04:27:37  robertj
+ * Added extra bullet proofing for some pathological conditions.
+ *
+ * Revision 1.58  2002/11/12 08:57:18  robertj
+ * Changed scope of PAbstraSortedList::Element class so descendant classes
+ *   can get at it.
+ * Fixed problem with GetValuesIndex not returning first element of a certain
+ *   value if the object was a PCaselessString.
+ *
+ * Revision 1.57  2002/06/25 02:24:59  robertj
+ * Improved assertion system to allow C++ class name to be displayed if
+ *   desired, especially relevant to container classes.
+ *
+ * Revision 1.56  2002/06/12 09:40:58  robertj
+ * Fixed printing of a dictionary to utilise the stream fill character between
+ *   each dictiionary element, as per general container semantics.
+ *
+ * Revision 1.55  2002/04/26 05:40:21  robertj
+ * Removed assumption that GetAt() on a dictionary will automatically convert
+ *   the index to a POrdinalyKey. This breaks the PCollection semantics for
+ *   GetAt() which is to get based on the ordinal position not the hashed position.
+ *
+ * Revision 1.54  2002/04/16 07:58:13  robertj
+ * Fixed MakeUnique for lists and sorted lists.
+ *
+ * Revision 1.53  2002/02/03 16:16:07  rogerh
+ * make nil static. Submitted by Peter Johnson <paj@chartermi.net>
+ *
+ * Revision 1.52  2002/01/31 05:02:50  robertj
+ * Fixed PSortedList::Remove function for removing objects with equal keys.
+ *
  * Revision 1.51  2001/06/07 04:49:26  robertj
  * Allowed for other separators than \n when printing elements of a collection.
  *
@@ -193,6 +270,8 @@
 
 #ifndef _WIN32_WCE
 #define new PNEW
+#undef  __CLASS__
+#define __CLASS__ GetClass()
 #endif
 
 
@@ -234,13 +313,14 @@ void PArrayObjects::CopyContents(const PArrayObjects & array)
 
 void PArrayObjects::DestroyContents()
 {
-  if (reference->deleteObjects) {
+  if (reference->deleteObjects && theArray != NULL) {
     for (PINDEX i = 0; i < theArray->GetSize(); i++) {
       if ((*theArray)[i] != NULL)
         delete (*theArray)[i];
     }
   }
   delete theArray;
+  theArray = NULL;
 }
 
 
@@ -264,7 +344,7 @@ void PArrayObjects::CloneContents(const PArrayObjects * array)
 
 PObject::Comparison PArrayObjects::Compare(const PObject & obj) const
 {
-  PAssert(obj.IsDescendant(PArrayObjects::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, PArrayObjects), PInvalidCast);
   const PArrayObjects & other = (const PArrayObjects &)obj;
   PINDEX i;
   for (i = 0; i < GetSize(); i++) {
@@ -398,6 +478,7 @@ void PAbstractList::DestroyContents()
 {
   RemoveAll();
   delete info;
+  info = NULL;
 }
 
 
@@ -412,10 +493,19 @@ void PAbstractList::CloneContents(const PAbstractList * list)
   Element * element = list->info->head;
 
   info = new Info;
-  PAssertNULL(info);
+  PAssert(info != NULL, POutOfMemory);
 
   while (element != NULL) {
-    Append(element->data->Clone());
+    Element * newElement = new Element(element->data->Clone());
+
+    if (info->head == NULL)
+      info->head = info->tail = newElement;
+    else {
+      newElement->prev = info->tail;
+      info->tail->next = newElement;
+      info->tail = newElement;
+    }
+
     element = element->next;
   }
 }
@@ -423,7 +513,7 @@ void PAbstractList::CloneContents(const PAbstractList * list)
 
 PObject::Comparison PAbstractList::Compare(const PObject & obj) const
 {
-  PAssert(obj.IsDescendant(PAbstractList::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, PAbstractList), PInvalidCast);
   Element * elmt1 = info->head;
   Element * elmt2 = ((const PAbstractList &)obj).info->head;
   while (elmt1 != NULL && elmt2 != NULL) {
@@ -450,7 +540,10 @@ BOOL PAbstractList::SetSize(PINDEX)
 
 PINDEX PAbstractList::Append(PObject * obj)
 {
-  Element * element = new Element(PAssertNULL(obj));
+  if (PAssertNULL(obj) == NULL)
+    return P_MAX_INDEX;
+
+  Element * element = new Element(obj);
   if (info->tail != NULL)
     info->tail->next = element;
   element->prev = info->tail;
@@ -467,7 +560,8 @@ PINDEX PAbstractList::Append(PObject * obj)
 
 PINDEX PAbstractList::Insert(const PObject & before, PObject * obj)
 {
-  PAssertNULL(obj);
+  if (PAssertNULL(obj) == NULL)
+    return P_MAX_INDEX;
   
   PINDEX where = GetObjectsIndex(&before);
   InsertAt(where, obj);
@@ -477,8 +571,9 @@ PINDEX PAbstractList::Insert(const PObject & before, PObject * obj)
 
 PINDEX PAbstractList::InsertAt(PINDEX index, PObject * obj)
 {
-  PAssertNULL(obj);
-
+  if (PAssertNULL(obj) == NULL)
+    return P_MAX_INDEX;
+  
   if (index >= GetSize())
     return Append(obj);
 
@@ -511,10 +606,23 @@ BOOL PAbstractList::Remove(const PObject * obj)
 
 PObject * PAbstractList::RemoveAt(PINDEX index)
 {
-  PAssert(SetCurrent(index), PInvalidArrayIndex);
+  if (!SetCurrent(index)) {
+    PAssertAlways(PInvalidArrayIndex);
+    return NULL;
+  }
 
+  if(info == NULL){
+    PAssertAlways("info is null");
+    return NULL;
+  }
+    
   Element * elmt = info->lastElement;
 
+  if(elmt == NULL){
+    PAssertAlways("elmt is null");
+    return NULL;
+  }
+  
   if (elmt->prev != NULL)
     elmt->prev->next = elmt->next;
   else {
@@ -537,6 +645,11 @@ PObject * PAbstractList::RemoveAt(PINDEX index)
     info->lastElement = elmt->prev;
     info->lastIndex--;
   }
+  
+  if((reference == NULL) || (reference->size == 0)){
+    PAssertAlways("reference is null or reference->size == 0");
+    return NULL;
+  }
   reference->size--;
 
   PObject * obj = elmt->data;
@@ -563,6 +676,18 @@ BOOL PAbstractList::SetAt(PINDEX index, PObject * val)
   return TRUE;
 }
 
+BOOL PAbstractList::ReplaceAt(PINDEX index, PObject * val)
+{
+  if (!SetCurrent(index))
+    return FALSE;
+  
+  if (info->lastElement->data != NULL && reference->deleteObjects) {
+    delete info->lastElement->data;
+  }
+
+  info->lastElement->data = val;
+  return TRUE;
+}
 
 PINDEX PAbstractList::GetObjectsIndex(const PObject * obj) const
 {
@@ -640,8 +765,6 @@ PAbstractList::Element::Element(PObject * theData)
 
 ///////////////////////////////////////////////////////////////////////////////
 
-PAbstractSortedList::Element nil = NULL;
-
 PAbstractSortedList::PAbstractSortedList()
 {
   info = new Info;
@@ -649,10 +772,23 @@ PAbstractSortedList::PAbstractSortedList()
 }
 
 
+PAbstractSortedList::Info::Info()
+{
+  root = &nil;
+  lastElement = NULL;
+  lastIndex = P_MAX_INDEX;
+  nil.parent = nil.left = nil.right = &nil;
+  nil.subTreeSize = 0;
+  nil.colour = Element::Black;
+  nil.data = NULL;
+}
+
+
 void PAbstractSortedList::DestroyContents()
 {
   RemoveAll();
   delete info;
+  info = NULL;
 }
 
 
@@ -664,16 +800,19 @@ void PAbstractSortedList::CopyContents(const PAbstractSortedList & list)
 
 void PAbstractSortedList::CloneContents(const PAbstractSortedList * list)
 {
-  Element * element = list->info->root;
-  while (element->left != &nil)
-    element = element->left;
+  Info * otherInfo = list->info;
 
   info = new Info;
-  PAssertNULL(info);
+  PAssert(info != NULL, POutOfMemory);
+  reference->size = 0;
 
-  while (element != &nil) {
+  // Have to do this in this manner rather than just doing a for() loop
+  // as "this" and "list" may be the same object and we just changed info in
+  // "this" so we need to use the info in "list" saved previously.
+  Element * element = otherInfo->OrderSelect(otherInfo->root, 1);
+  while (element != &otherInfo->nil) {
     Append(element->data->Clone());
-    element = element->Successor();
+    element = otherInfo->Successor(element);
   }
 }
 
@@ -686,26 +825,26 @@ BOOL PAbstractSortedList::SetSize(PINDEX)
 
 PObject::Comparison PAbstractSortedList::Compare(const PObject & obj) const
 {
-  PAssert(obj.IsDescendant(PAbstractSortedList::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, PAbstractSortedList), PInvalidCast);
   Element * elmt1 = info->root;
-  while (elmt1->left != &nil)
+  while (elmt1->left != &info->nil)
     elmt1 = elmt1->left;
 
   Element * elmt2 = ((const PAbstractSortedList &)obj).info->root;
-  while (elmt2->left != &nil)
+  while (elmt2->left != &info->nil)
     elmt2 = elmt2->left;
 
-  while (elmt1 != &nil && elmt2 != &nil) {
-    if (elmt1 == &nil)
+  while (elmt1 != &info->nil && elmt2 != &info->nil) {
+    if (elmt1 == &info->nil)
       return LessThan;
-    if (elmt2 == &nil)
+    if (elmt2 == &info->nil)
       return GreaterThan;
     if (*elmt1->data < *elmt2->data)
       return LessThan;
     if (*elmt1->data > *elmt2->data)
       return GreaterThan;
-    elmt1 = elmt1->Successor();
-    elmt2 = elmt2->Successor();
+    elmt1 = info->Successor(elmt1);
+    elmt2 = info->Successor(elmt2);
   }
   return EqualTo;
 }
@@ -713,16 +852,24 @@ PObject::Comparison PAbstractSortedList::Compare(const PObject & obj) const
 
 PINDEX PAbstractSortedList::Append(PObject * obj)
 {
-  Element * z = new Element(PAssertNULL(obj));
+  if (PAssertNULL(obj) == NULL)
+    return P_MAX_INDEX;
+
+  Element * z = new Element;
+  z->parent = z->left = z->right = &info->nil;
+  z->colour = Element::Black;
+  z->subTreeSize = 1;
+  z->data = obj;
+
   Element * x = info->root;
-  Element * y = &nil;
-  while (x != &nil) {
+  Element * y = &info->nil;
+  while (x != &info->nil) {
     x->subTreeSize++;
     y = x;
     x = *z->data < *x->data ? x->left : x->right;
   }
   z->parent = y;
-  if (y == &nil)
+  if (y == &info->nil)
     info->root = z;
   else if (*z->data < *y->data)
     y->left = z;
@@ -788,20 +935,20 @@ PINDEX PAbstractSortedList::Append(PObject * obj)
 
 BOOL PAbstractSortedList::Remove(const PObject * obj)
 {
-  Element * element = info->root;
-  while (element != &nil && element->data != obj)
-    element = *obj < *element->data ? element->left : element->right;
-  if (element == &nil)
+  if (GetObjectsIndex(obj) == P_MAX_INDEX)
     return FALSE;
 
-  RemoveElement(element);
+  RemoveElement(info->lastElement);
   return TRUE;
 }
 
 
 PObject * PAbstractSortedList::RemoveAt(PINDEX index)
 {
-  Element * node = info->root->OrderSelect(index+1);
+  Element * node = info->OrderSelect(info->root, index+1);
+  if (node == &info->nil)
+    return NULL;
+
   PObject * data = node->data;
   RemoveElement(node);
   return reference->deleteObjects ? (PObject *)NULL : data;
@@ -810,10 +957,10 @@ PObject * PAbstractSortedList::RemoveAt(PINDEX index)
 
 void PAbstractSortedList::RemoveAll()
 {
-  if (info->root != &nil) {
-    info->root->DeleteSubTrees(reference->deleteObjects);
+  if (info->root != &info->nil) {
+    DeleteSubTrees(info->root, reference->deleteObjects);
     delete info->root;
-    info->root = &nil;
+    info->root = &info->nil;
     reference->size = 0;
   }
 }
@@ -845,15 +992,15 @@ PObject * PAbstractSortedList::GetAt(PINDEX index) const
   if (index != info->lastIndex) {
     if (index == info->lastIndex-1) {
       info->lastIndex--;
-      info->lastElement = info->lastElement->Predecessor();
+      info->lastElement = info->Predecessor(info->lastElement);
     }
     else if (index == info->lastIndex+1 && info->lastElement != NULL) {
       info->lastIndex++;
-      info->lastElement = info->lastElement->Successor();
+      info->lastElement = info->Successor(info->lastElement);
     }
     else {
       info->lastIndex = index;
-      info->lastElement = info->root->OrderSelect(index+1);
+      info->lastElement = info->OrderSelect(info->root, index+1);
     }
   }
 
@@ -864,7 +1011,7 @@ PObject * PAbstractSortedList::GetAt(PINDEX index) const
 PINDEX PAbstractSortedList::GetObjectsIndex(const PObject * obj) const
 {
   Element * elmt = NULL;
-  PINDEX pos = info->root->ValueSelect(*obj, elmt);
+  PINDEX pos = ValueSelect(info->root, *obj, (const Element **)&elmt);
   if (pos == P_MAX_INDEX)
     return P_MAX_INDEX;
 
@@ -872,14 +1019,14 @@ PINDEX PAbstractSortedList::GetObjectsIndex(const PObject * obj) const
     PINDEX savePos = pos;
     Element * saveElmt = elmt;
     while (elmt->data != obj &&
-            (elmt = elmt->Predecessor()) != &nil &&
+            (elmt = info->Predecessor(elmt)) != &info->nil &&
             *obj == *elmt->data)
       pos--;
     if (elmt->data != obj) {
       pos = savePos;
       elmt = saveElmt;
       while (elmt->data != obj &&
-              (elmt = elmt->Successor()) != &nil &&
+              (elmt = info->Successor(elmt)) != &info->nil &&
               *obj == *elmt->data)
         pos++;
       if (elmt->data != obj)
@@ -896,14 +1043,15 @@ PINDEX PAbstractSortedList::GetObjectsIndex(const PObject * obj) const
 
 PINDEX PAbstractSortedList::GetValuesIndex(const PObject & obj) const
 {
-  PINDEX pos = info->root->ValueSelect(obj, info->lastElement);
+  PINDEX pos = ValueSelect(info->root, obj, (const Element **)&info->lastElement);
   if (pos == P_MAX_INDEX)
     return P_MAX_INDEX;
 
   info->lastIndex = pos;
 
   Element * prev;
-  while ((prev = info->lastElement->Predecessor()) != &nil && obj == *prev->data) {
+  while ((prev = info->Predecessor(info->lastElement)) != &info->nil &&
+                                  prev->data->Compare(obj) == EqualTo) {
     info->lastElement = prev;
     info->lastIndex--;
   }
@@ -914,23 +1062,25 @@ PINDEX PAbstractSortedList::GetValuesIndex(const PObject & obj) const
 
 void PAbstractSortedList::RemoveElement(Element * node)
 {
-  PAssertNULL(node);
+  // Don't try an remove one of the special leaf nodes!
+  if (PAssertNULL(node) == &info->nil)
+    return;
 
   if (node->data != NULL && reference->deleteObjects)
     delete node->data;
 
-  Element * y = node->left == &nil || node->right == &nil ? node : node->Successor();
+  Element * y = node->left == &info->nil || node->right == &info->nil ? node : info->Successor(node);
 
   Element * t = y;
-  while (t != &nil) {
+  while (t != &info->nil) {
     t->subTreeSize--;
     t = t->parent;
   }
 
-  Element * x = y->left != &nil ? y->left : y->right;
+  Element * x = y->left != &info->nil ? y->left : y->right;
   x->parent = y->parent;
 
-  if (y->parent == &nil)
+  if (y->parent == &info->nil)
     info->root = x;
   else if (y == y->parent->left)
     y->parent->left = x;
@@ -1010,10 +1160,10 @@ void PAbstractSortedList::LeftRotate(Element * node)
 {
   Element * pivot = PAssertNULL(node)->right;
   node->right = pivot->left;
-  if (pivot->left != &nil)
+  if (pivot->left != &info->nil)
     pivot->left->parent = node;
   pivot->parent = node->parent;
-  if (node->parent == &nil)
+  if (node->parent == &info->nil)
     info->root = pivot;
   else if (node == node->parent->left)
     node->parent->left = pivot;
@@ -1030,10 +1180,10 @@ void PAbstractSortedList::RightRotate(Element * node)
 {
   Element * pivot = PAssertNULL(node)->left;
   node->left = pivot->right;
-  if (pivot->right != &nil)
+  if (pivot->right != &info->nil)
     pivot->right->parent = node;
   pivot->parent = node->parent;
-  if (node->parent == &nil)
+  if (node->parent == &info->nil)
     info->root = pivot;
   else if (node == node->parent->right)
     node->parent->right = pivot;
@@ -1046,26 +1196,16 @@ void PAbstractSortedList::RightRotate(Element * node)
 }
 
 
-PAbstractSortedList::Element::Element(PObject * theData)
-{
-  parent = left = right = &nil;
-  colour = Black;
-  subTreeSize = theData != NULL ? 1 : 0;
-  data = theData;
-}
-
-
-PAbstractSortedList::Element * PAbstractSortedList::Element::Successor() const
+PAbstractSortedList::Element * PAbstractSortedList::Info ::Successor(const Element * node) const
 {
   Element * next;
-  if (right != &nil) {
-    next = right;
+  if (node->right != &nil) {
+    next = node->right;
     while (next->left != &nil)
       next = next->left;
   }
   else {
-    next = parent;
-    const Element * node = this;
+    next = node->parent;
     while (next != &nil && node == next->right) {
       node = next;
       next = node->parent;
@@ -1075,17 +1215,16 @@ PAbstractSortedList::Element * PAbstractSortedList::Element::Successor() const
 }
 
 
-PAbstractSortedList::Element*PAbstractSortedList::Element::Predecessor() const
+PAbstractSortedList::Element * PAbstractSortedList::Info ::Predecessor(const Element * node) const
 {
   Element * pred;
-  if (left != &nil) {
-    pred = left;
+  if (node->left != &nil) {
+    pred = node->left;
     while (pred->right != &nil)
       pred = pred->right;
   }
   else {
-    pred = parent;
-    const Element * node = this;
+    pred = node->parent;
     while (pred != &nil && node == pred->left) {
       node = pred;
       pred = node->parent;
@@ -1095,45 +1234,46 @@ PAbstractSortedList::Element*PAbstractSortedList::Element::Predecessor() const
 }
 
 
-PAbstractSortedList::Element * PAbstractSortedList::Element::OrderSelect(PINDEX index)
+PAbstractSortedList::Element * PAbstractSortedList::Info::OrderSelect(Element * node, PINDEX index) const
 {
-  PINDEX r = left->subTreeSize+1;
+  PINDEX r = node->left->subTreeSize+1;
   if (index == r)
-    return this;
+    return node;
 
   if (index < r) {
-    if (left != &nil)
-      return left->OrderSelect(index);
+    if (node->left != &nil)
+      return OrderSelect(node->left, index);
   }
   else {
-    if (right != &nil)
-      return right->OrderSelect(index - r);
+    if (node->right != &nil)
+      return OrderSelect(node->right, index - r);
   }
 
-  PAssertAlways("Order select failed!");
-  return &nil;
+  PAssertAlways2("PAbstractSortedList::Element", "Order select failed!");
+  return (Element *)&nil;
 }
 
 
-PINDEX PAbstractSortedList::Element::ValueSelect(const PObject & obj,
-                                                 Element * & lastElement)
+PINDEX PAbstractSortedList::ValueSelect(const Element * node,
+                                        const PObject & obj,
+                                        const Element ** lastElement) const
 {
-  if (this != &nil) {
-    switch (data->Compare(obj)) {
+  if (node != &info->nil) {
+    switch (node->data->Compare(obj)) {
       case PObject::LessThan :
       {
-        PINDEX index = right->ValueSelect(obj, lastElement);
+        PINDEX index = ValueSelect(node->right, obj, lastElement);
         if (index != P_MAX_INDEX)
-          return left->subTreeSize + index + 1;
+          return node->left->subTreeSize + index + 1;
         break;
       }
 
       case PObject::GreaterThan :
-        return left->ValueSelect(obj, lastElement);
+        return ValueSelect(node->left, obj, lastElement);
 
       default :
-        lastElement = this;
-        return left->subTreeSize;
+        *lastElement = node;
+        return node->left->subTreeSize;
     }
   }
 
@@ -1141,33 +1281,25 @@ PINDEX PAbstractSortedList::Element::ValueSelect(const PObject & obj,
 }
 
 
-void PAbstractSortedList::Element::DeleteSubTrees(BOOL deleteObject)
+void PAbstractSortedList::DeleteSubTrees(Element * node, BOOL deleteObject)
 {
-  if (left != &nil) {
-    left->DeleteSubTrees(deleteObject);
-    delete left;
-    left = &nil;
+  if (node->left != &info->nil) {
+    DeleteSubTrees(node->left, deleteObject);
+    delete node->left;
+    node->left = &info->nil;
   }
-  if (right != &nil) {
-    right->DeleteSubTrees(deleteObject);
-    delete right;
-    right = &nil;
+  if (node->right != &info->nil) {
+    DeleteSubTrees(node->right, deleteObject);
+    delete node->right;
+    node->right = &info->nil;
   }
   if (deleteObject) {
-    delete data;
-    data = NULL;
+    delete node->data;
+    node->data = NULL;
   }
 }
 
 
-PAbstractSortedList::Info::Info()
-{
-  root = &nil;
-  lastElement = NULL;
-  lastIndex = P_MAX_INDEX;
-}
-
-
 ///////////////////////////////////////////////////////////////////////////////
 
 PObject * POrdinalKey::Clone() const
@@ -1178,7 +1310,7 @@ PObject * POrdinalKey::Clone() const
 
 PObject::Comparison POrdinalKey::Compare(const PObject & obj) const
 {
-  PAssert(obj.IsDescendant(POrdinalKey::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, POrdinalKey), PInvalidCast);
   const POrdinalKey & other = (const POrdinalKey &)obj;
   
   if (theKey < other.theKey)
@@ -1233,7 +1365,7 @@ PINDEX PHashTable::Table::AppendElement(PObject * key, PObject * data)
   PINDEX bucket = PAssertNULL(key)->HashFunction();
   Element * list = GetAt(bucket);
   Element * element = new Element;
-  PAssertNULL(element);
+  PAssert(element != NULL, POutOfMemory);
   element->key = key;
   element->data = data;
   if (list == NULL) {
@@ -1371,15 +1503,18 @@ PINDEX PHashTable::Table::GetElementsIndex(
 PHashTable::PHashTable()
   : hashTable(new PHashTable::Table)
 {
-  PAssertNULL(hashTable);
+  PAssert(hashTable != NULL, POutOfMemory);
   hashTable->lastElement = NULL;
 }
 
 
 void PHashTable::DestroyContents()
 {
-  hashTable->reference->deleteObjects = reference->deleteObjects;
-  delete hashTable;
+  if (hashTable != NULL) {
+    hashTable->reference->deleteObjects = reference->deleteObjects;
+    delete hashTable;
+    hashTable = NULL;
+  }
 }
 
 
@@ -1391,13 +1526,11 @@ void PHashTable::CopyContents(const PHashTable & hash)
   
 void PHashTable::CloneContents(const PHashTable * hash)
 {
-  PAssertNULL(hash);
-  PINDEX sz = hash->GetSize();
-  PAssertNULL(hash->hashTable);
-  PHashTable::Table * original = hash->hashTable;
+  PINDEX sz = PAssertNULL(hash)->GetSize();
+  PHashTable::Table * original = PAssertNULL(hash->hashTable);
 
   hashTable = new PHashTable::Table(original->GetSize());
-  PAssertNULL(hashTable);
+  PAssert(hashTable != NULL, POutOfMemory);
   hashTable->lastElement = NULL;
 
   for (PINDEX i = 0; i < sz; i++) {
@@ -1412,7 +1545,7 @@ void PHashTable::CloneContents(const PHashTable * hash)
 
 PObject::Comparison PHashTable::Compare(const PObject & obj) const
 {
-  PAssert(obj.IsDescendant(PHashTable::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, PHashTable), PInvalidCast);
   return reference != ((const PHashTable &)obj).reference
                                                       ? GreaterThan : EqualTo;
 }
@@ -1484,7 +1617,10 @@ PINDEX PAbstractSet::InsertAt(PINDEX, PObject * obj)
 
 BOOL PAbstractSet::Remove(const PObject * obj)
 {
-  if (hashTable->GetElementAt(*PAssertNULL(obj)) == NULL)
+  if (PAssertNULL(obj) == NULL)
+    return FALSE;
+
+  if (hashTable->GetElementAt(*obj) == NULL)
     return FALSE;
 
   hashTable->deleteKeys = hashTable->reference->deleteObjects = reference->deleteObjects;
@@ -1554,10 +1690,14 @@ PINDEX PAbstractDictionary::InsertAt(PINDEX index, PObject * obj)
 }
  
  
-BOOL PAbstractDictionary::Remove(const PObject *)
+BOOL PAbstractDictionary::Remove(const PObject * obj)
 {
-  PAssertAlways(PUnimplementedFunction);
-  return FALSE;
+  PINDEX idx = GetObjectsIndex(obj);
+  if (idx == P_MAX_INDEX)
+    return FALSE;
+
+  RemoveAt(idx);
+  return TRUE;
 }
 
 
@@ -1583,13 +1723,14 @@ PINDEX PAbstractDictionary::GetValuesIndex(const PObject & obj) const
 
 BOOL PAbstractDictionary::SetAt(PINDEX index, PObject * val)
 {
-  return AbstractSetAt(POrdinalKey(index), val);
+  return AbstractSetAt(AbstractGetKeyAt(index), val);
 }
 
 
 PObject * PAbstractDictionary::GetAt(PINDEX index) const
 {
-  return AbstractGetAt(POrdinalKey(index));
+  PAssert(hashTable->SetLastElementAt(index), PInvalidArrayIndex);
+  return hashTable->lastElement->data;
 }
  
  
@@ -1616,7 +1757,7 @@ BOOL PAbstractDictionary::AbstractSetAt(const PObject & key, PObject * obj)
       reference->size++;
     }
     else {
-      if (reference->deleteObjects)
+      if ((reference->deleteObjects) && (hashTable->lastElement->data != obj)) 
         delete hashTable->lastElement->data;
       hashTable->lastElement->data = obj;
     }
@@ -1641,8 +1782,18 @@ PObject & PAbstractDictionary::GetRefAt(const PObject & key) const
 
 void PAbstractDictionary::PrintOn(ostream &strm) const
 {
-  for (PINDEX  i = 0; i < GetSize(); i++)
-    strm << AbstractGetKeyAt(i) << '=' << AbstractGetDataAt(i) << endl;
+  char separator = strm.fill();
+  if (separator == ' ')
+    separator = '\n';
+
+  for (PINDEX i = 0; i < GetSize(); i++) {
+    if (i > 0)
+      strm << separator;
+    strm << AbstractGetKeyAt(i) << '=' << AbstractGetDataAt(i);
+  }
+
+  if (separator == '\n')
+    strm << separator;
 }
 
 
