@@ -27,6 +27,79 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: pchannel.cxx,v $
+ * Revision 1.36  2005/11/30 12:47:42  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.35  2005/10/06 08:20:14  csoutheren
+ * Changed WriteString to ensure it always writes all data even with partial writes
+ *
+ * Revision 1.34  2005/09/18 11:05:36  dominance
+ * include/ptlib/channel.h, include/ptlib/pstring.h, src/ptlib/common/contain.cxx,
+ * src/ptlib/common/pchannel.cxx:
+ * correct the STL defined checking to use proper syntax.
+ *
+ * include/ptlib/object.h:
+ * re-add typedef to compile on mingw
+ *
+ * make/ptlib-config.in:
+ * import a long-standing fix from the Debian packs which allows usage of
+ * ptlib-config without manually adding -lpt for each of the subsequent
+ * projects
+ *
+ * Revision 1.33  2005/08/05 20:41:42  csoutheren
+ * Added unix support for scattered read/write
+ *
+ * Revision 1.32  2004/07/03 03:00:46  rjongbloed
+ * Fixed MSVC warning
+ *
+ * Revision 1.31  2004/07/03 01:48:28  rjongbloed
+ * Fixed memory leak caused by buggy iostream, can't do init twice. Thanks Norbert Bartalsky
+ *
+ * Revision 1.30  2004/06/08 01:31:08  csoutheren
+ * Make the test sense correct for the init(NULL)
+ *
+ * Revision 1.29  2004/06/08 01:29:00  csoutheren
+ * Removed memory leak on VS.net caused by unobvious iostream allocation
+ *
+ * Revision 1.28  2004/04/09 06:38:11  rjongbloed
+ * Fixed compatibility with STL based streams, eg as used by VC++2003
+ *
+ * Revision 1.27  2004/04/03 08:22:21  csoutheren
+ * Remove pseudo-RTTI and replaced with real RTTI
+ *
+ * Revision 1.26  2004/03/22 10:15:28  rjongbloed
+ * Added classes similar to PWaitAndSignal to automatically unlock a PReadWriteMutex
+ *   when goes out of scope.
+ *
+ * Revision 1.25  2004/02/24 11:19:32  rjongbloed
+ * Fixed seekpos() function on channel to read data when seeking beyond end of corrent stream.
+ *
+ * Revision 1.24  2003/12/19 04:31:27  csoutheren
+ * Changed GetLastReadCount and GetLastWriteCount to be virtual
+ *
+ * Revision 1.23  2003/04/23 00:37:04  craigs
+ * More casts to avoid problems on MacOSX thanks to Shawn Hsiao
+ *
+ * Revision 1.22  2003/03/19 00:10:24  robertj
+ * Added ability to use seekoff() in a PChannel streambuf that is not a file.
+ *
+ * Revision 1.21  2003/02/11 07:22:43  robertj
+ * Fixed strange behaviour in ReadString(P_MAX_INDEX) with DOS text files where
+ *   it would get extra garbage at the end of the string, thanks Joerg Schoemer.
+ *
+ * Revision 1.20  2003/02/10 01:01:03  robertj
+ * Fixed portability issue for lseek() calls, should just look for -1 return
+ *   value to indicate error, thanks Joerg Schoemer
+ *
+ * Revision 1.19  2002/12/19 03:37:05  craigs
+ * Simplified PChannel::WriteString
+ *
+ * Revision 1.18  2002/04/09 02:30:18  robertj
+ * Removed GCC3 variable as __GNUC__ can be used instead, thanks jason Spence
+ *
+ * Revision 1.17  2002/01/26 23:57:45  craigs
+ * Changed for GCC 3.0 compatibility, thanks to manty@manty.net
+ *
  * Revision 1.16  2001/11/13 04:13:22  robertj
  * Added ability to adjust size of ios buffer on PChannels.
  *
@@ -151,7 +224,7 @@ int PChannelStreamBuffer::sync()
   int inAvail = egptr() - gptr();
   if (inAvail > 0) {
     setg(eback(), egptr(), egptr());
-    if (channel->IsDescendant(PFile::Class()))
+    if (PIsDescendant(channel, PFile))
       ((PFile *)channel)->SetPosition(-inAvail, PFile::Current);
   }
 
@@ -162,45 +235,77 @@ int PChannelStreamBuffer::sync()
 }
 
 
-streampos PChannelStreamBuffer::seekoff(streamoff off,
-#ifdef __MWERKS__
-                                        ios::seekdir dir, ios::openmode)
+#ifdef __USE_STL__
+streampos PChannelStreamBuffer::seekoff(off_type off, ios_base::seekdir dir, ios_base::openmode)
 #else
-                                        ios::seek_dir dir, int)
+streampos PChannelStreamBuffer::seekoff(streamoff off, ios::seek_dir dir, int)
 #endif
 {
   sync();
-  if (!channel->IsDescendant(PFile::Class()))
-    return -1;
-  ((PFile *)channel)->SetPosition(off, (PFile::FilePositionOrigin)dir);
-  return ((PFile *)channel)->GetPosition();
+  if (PIsDescendant(channel, PFile)) {
+    PFile * file = (PFile *)channel;
+    file->SetPosition(off, (PFile::FilePositionOrigin)dir);
+    return file->GetPosition();
+  }
+
+  // If we have an input stream and the buffer is empty then force a read so
+  // we can seek ahead.
+  if (egptr() == gptr()) {
+    int c = underflow();
+    if (c == EOF)
+      return EOF;
+  }
+
+  while (off-- > 0) {
+    if (sbumpc() == EOF)
+      return EOF;
+  }
+    
+  return egptr() - gptr();
 }
 
 
+#ifdef __USE_STL__
+streampos PChannelStreamBuffer::seekpos(pos_type pos, ios_base::openmode mode)
+{
+  return seekoff(pos, ios_base::beg, mode);
+}
+#endif
+
+
+#ifdef _MSC_VER
+#pragma warning(disable:4355)
+#endif
+
 PChannel::PChannel()
-  : readTimeout(PMaxTimeInterval), writeTimeout(PMaxTimeInterval)
+  : iostream(new PChannelStreamBuffer(this)),
+    readTimeout(PMaxTimeInterval), writeTimeout(PMaxTimeInterval)
 {
   os_handle = -1;
   memset(lastErrorCode, 0, sizeof(lastErrorCode));
   memset(lastErrorNumber, 0, sizeof(lastErrorNumber));
   lastReadCount = lastWriteCount = 0;
-  init(new PChannelStreamBuffer(this));
   Construct();
 }
 
+#ifdef _MSC_VER
+#pragma warning(default:4355)
+#endif
+
 
 PChannel::~PChannel()
 {
   flush();
   Close();
   delete (PChannelStreamBuffer *)rdbuf();
+#ifndef _WIN32
   init(NULL);
+#endif
 }
 
-
 PObject::Comparison PChannel::Compare(const PObject & obj) const
 {
-  PAssert(IsDescendant(PChannel::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, PChannel), PInvalidCast);
   int h1 = GetHandle();
   int h2 = ((const PChannel&)obj).GetHandle();
   if (h1 < h2)
@@ -222,6 +327,15 @@ BOOL PChannel::IsOpen() const
   return os_handle >= 0;
 }
 
+PINDEX PChannel::GetLastReadCount() const
+{ 
+  return lastReadCount; 
+}
+
+PINDEX PChannel::GetLastWriteCount() const
+{ 
+  return lastWriteCount; 
+}
 
 int PChannel::ReadChar()
 {
@@ -270,10 +384,16 @@ PString PChannel::ReadString(PINDEX len)
       l += lastReadCount;
     }
     str.SetSize(l+1);
-  } else {
+
+    /*Need to put in a null at the end to allow for MSDOS/Win32 text files
+      which returns fewer bytes than actually read as it shrinks the data into
+      the removed carriage returns, so it actually changes the buffer beyond
+      what it indicated. */
+    str[l] = '\0';
+  }
+  else {
     if (!ReadBlock(str.GetPointer(len+1), len))
-      return PString();
-    str.SetSize(lastReadCount+1);
+      return PString::Empty();
   }
 
   return str;
@@ -282,13 +402,17 @@ PString PChannel::ReadString(PINDEX len)
 
 BOOL PChannel::WriteString(const PString & str)
 {
-  const char * ptr = str;
-  PINDEX len = 0, slen = str.GetLength();
-
-  while (len < slen && Write(ptr+len, slen - len))
-    len += lastWriteCount;
-
-  return len == slen;
+  PINDEX len = str.GetLength();
+  PINDEX written = 0;
+  while (written < len) {
+    if (!Write((const char *)str + written, len - written)) {
+      lastWriteCount += written;
+      return FALSE;
+    }
+    written += lastWriteCount;
+  }
+  lastWriteCount = written;
+  return TRUE;
 }
 
 
@@ -558,6 +682,41 @@ BOOL PChannel::SetErrorValues(Errors errorCode, int errorNum, ErrorGroup group)
   return errorCode == NoError;
 }
 
+#ifndef P_HAS_RECVMSG
+
+BOOL PChannel::Read(const VectorOfSlice & slices)
+{
+  PINDEX length = 0;
+
+  VectorOfSlice::const_iterator r;
+  for (r = slices.begin(); r != slices.end(); ++r) {
+    BOOL stat = Read(r->iov_base, r->iov_len);
+    length        += lastReadCount;
+    lastReadCount = length;
+    if (!stat)
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
+BOOL PChannel::Write(const VectorOfSlice & slices)
+{
+  PINDEX length = 0;
+
+  VectorOfSlice::const_iterator r;
+  for (r = slices.begin(); r != slices.end(); ++r) {
+    BOOL stat = Write(r->iov_base, r->iov_len);
+    length        += lastWriteCount;
+    lastWriteCount = length;
+    if (!stat)
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
+#endif // P_HAS_RECVMSG
 
 ///////////////////////////////////////////////////////////////////////////////
 // PIndirectChannel
@@ -571,7 +730,7 @@ PIndirectChannel::PIndirectChannel()
 
 PObject::Comparison PIndirectChannel::Compare(const PObject & obj) const
 {
-  PAssert(obj.IsDescendant(PIndirectChannel::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, PIndirectChannel), PInvalidCast);
   const PIndirectChannel & other = (const PIndirectChannel &)obj;
   return readChannel == other.readChannel &&
          writeChannel == other.writeChannel ? EqualTo : GreaterThan;
@@ -580,22 +739,20 @@ PObject::Comparison PIndirectChannel::Compare(const PObject & obj) const
 
 PString PIndirectChannel::GetName() const
 {
-  ((PIndirectChannel*)this)->channelPointerMutex.StartRead();
+  PReadWaitAndSignal mutex(channelPointerMutex);
 
-  PString name;
   if (readChannel != NULL && readChannel == writeChannel)
-    name = readChannel->GetName();
-  else {
-    name = "R<";
-    if (readChannel != NULL)
-      name += readChannel->GetName();
-    name += "> T<";
-    if (writeChannel != NULL)
-      name += writeChannel->GetName();
-    name += ">";
-  }
+    return readChannel->GetName();
 
-  ((PIndirectChannel*)this)->channelPointerMutex.EndRead();
+  PStringStream name;
+
+  name << "R<";
+  if (readChannel != NULL)
+    name << readChannel->GetName();
+  name << "> T<";
+  if (writeChannel != NULL)
+    name << writeChannel->GetName();
+  name << '>';
 
   return name;
 }
@@ -639,18 +796,15 @@ BOOL PIndirectChannel::Close()
 
 BOOL PIndirectChannel::IsOpen() const
 {
-  ((PIndirectChannel*)this)->channelPointerMutex.StartRead();
+  PReadWaitAndSignal mutex(channelPointerMutex);
 
-  BOOL returnValue;
   if (readChannel != NULL && readChannel == writeChannel)
-    returnValue = readChannel->IsOpen();
-  else {
-    returnValue = readChannel != NULL ? readChannel->IsOpen() : FALSE;
-    if (writeChannel != NULL)
-      returnValue = writeChannel->IsOpen() || returnValue;
-  }
+    return readChannel->IsOpen();
 
-  ((PIndirectChannel*)this)->channelPointerMutex.EndRead();
+  BOOL returnValue = readChannel != NULL ? readChannel->IsOpen() : FALSE;
+
+  if (writeChannel != NULL)
+    returnValue = writeChannel->IsOpen() || returnValue;
 
   return returnValue;
 }
@@ -660,25 +814,20 @@ BOOL PIndirectChannel::Read(void * buf, PINDEX len)
 {
   flush();
 
-  channelPointerMutex.StartRead();
+  PReadWaitAndSignal mutex(channelPointerMutex);
 
-  BOOL returnValue;
   if (readChannel == NULL) {
     SetErrorValues(NotOpen, EBADF, LastReadError);
-    returnValue = FALSE;
-  }
-  else {
-    readChannel->SetReadTimeout(readTimeout);
-
-    returnValue = readChannel->Read(buf, len);
-
-    SetErrorValues(readChannel->GetErrorCode(LastReadError),
-                   readChannel->GetErrorNumber(LastReadError),
-                   LastReadError);
-    lastReadCount = readChannel->GetLastReadCount();
+    return FALSE;
   }
 
-  channelPointerMutex.EndRead();
+  readChannel->SetReadTimeout(readTimeout);
+  BOOL returnValue = readChannel->Read(buf, len);
+
+  SetErrorValues(readChannel->GetErrorCode(LastReadError),
+                 readChannel->GetErrorNumber(LastReadError),
+                 LastReadError);
+  lastReadCount = readChannel->GetLastReadCount();
 
   return returnValue;
 }
@@ -688,26 +837,21 @@ BOOL PIndirectChannel::Write(const void * buf, PINDEX len)
 {
   flush();
 
-  channelPointerMutex.StartRead();
+  PReadWaitAndSignal mutex(channelPointerMutex);
 
-  BOOL returnValue;
   if (writeChannel == NULL) {
     SetErrorValues(NotOpen, EBADF, LastWriteError);
-    returnValue = FALSE;
-  }
-  else {
-    writeChannel->SetWriteTimeout(writeTimeout);
-
-    returnValue = writeChannel->Write(buf, len);
-
-    SetErrorValues(writeChannel->GetErrorCode(LastWriteError),
-                   writeChannel->GetErrorNumber(LastWriteError),
-                   LastWriteError);
-
-    lastWriteCount = writeChannel->GetLastWriteCount();
+    return FALSE;
   }
 
-  channelPointerMutex.EndRead();
+  writeChannel->SetWriteTimeout(writeTimeout);
+  BOOL returnValue = writeChannel->Write(buf, len);
+
+  SetErrorValues(writeChannel->GetErrorCode(LastWriteError),
+                 writeChannel->GetErrorNumber(LastWriteError),
+                 LastWriteError);
+
+  lastWriteCount = writeChannel->GetLastWriteCount();
 
   return returnValue;
 }
@@ -715,18 +859,15 @@ BOOL PIndirectChannel::Write(const void * buf, PINDEX len)
 
 BOOL PIndirectChannel::Shutdown(ShutdownValue value)
 {
-  channelPointerMutex.StartRead();
+  PReadWaitAndSignal mutex(channelPointerMutex);
 
-  BOOL returnValue;
   if (readChannel != NULL && readChannel == writeChannel)
-    returnValue = readChannel->Shutdown(value);
-  else {
-    returnValue = readChannel != NULL ? readChannel->Shutdown(value) : FALSE;
-    if (writeChannel != NULL)
-      returnValue = writeChannel->Shutdown(value) || returnValue;
-  }
+    return readChannel->Shutdown(value);
 
-  channelPointerMutex.EndRead();
+  BOOL returnValue = readChannel != NULL ? readChannel->Shutdown(value) : FALSE;
+
+  if (writeChannel != NULL)
+    returnValue = writeChannel->Shutdown(value) || returnValue;
 
   return returnValue;
 }
@@ -746,7 +887,7 @@ PString PIndirectChannel::GetErrorText(ErrorGroup group) const
 
 BOOL PIndirectChannel::Open(PChannel & channel)
 {
-  return Open(&channel, FALSE);
+  return Open(&channel, (BOOL)FALSE);
 }
 
 
@@ -829,25 +970,15 @@ BOOL PIndirectChannel::SetWriteChannel(PChannel * channel, BOOL autoDelete)
 
 PChannel * PIndirectChannel::GetBaseReadChannel() const
 {
-  ((PIndirectChannel*)this)->channelPointerMutex.StartRead();
-
-  PChannel * returnValue = readChannel != NULL ? readChannel->GetBaseReadChannel() : 0;
-
-  ((PIndirectChannel*)this)->channelPointerMutex.EndRead();
-
-  return returnValue;
+  PReadWaitAndSignal mutex(channelPointerMutex);
+  return readChannel != NULL ? readChannel->GetBaseReadChannel() : 0;
 }
 
 
 PChannel * PIndirectChannel::GetBaseWriteChannel() const
 {
-  ((PIndirectChannel*)this)->channelPointerMutex.StartRead();
-
-  PChannel * returnValue = writeChannel != NULL ? writeChannel->GetBaseWriteChannel() : 0;
-
-  ((PIndirectChannel*)this)->channelPointerMutex.EndRead();
-
-  return returnValue;
+  PReadWaitAndSignal mutex(channelPointerMutex);
+  return writeChannel != NULL ? writeChannel->GetBaseWriteChannel() : 0;
 }
 
 
@@ -862,7 +993,7 @@ PFile::~PFile()
 
 PObject::Comparison PFile::Compare(const PObject & obj) const
 {
-  PAssert(obj.IsDescendant(PFile::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, PFile), PInvalidCast);
   return path.Compare(((const PFile &)obj).path);
 }
 
@@ -940,7 +1071,7 @@ off_t PFile::GetLength() const
 #else
   off_t pos = _lseek(GetHandle(), 0, SEEK_CUR);
   off_t len = _lseek(GetHandle(), 0, SEEK_END);
-  PAssertOS(_lseek(GetHandle(), pos, SEEK_SET) == pos);
+  PAssertOS(_lseek(GetHandle(), pos, SEEK_SET) != (off_t)-1);
   return len;
 #endif
 }
@@ -958,7 +1089,7 @@ BOOL PFile::SetPosition(off_t pos, FilePositionOrigin origin)
 #ifdef WOT_NO_FILESYSTEM
   return TRUE;
 #else
-  return _lseek(GetHandle(), pos, origin) == pos;
+  return _lseek(GetHandle(), pos, origin) != (off_t)-1;
 #endif
 }
 
