@@ -27,6 +27,29 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: ptime.cxx,v $
+ * Revision 1.47  2004/06/06 08:50:08  rjongbloed
+ * Fixed rounding error in microsoeconds stream output, was not properly cascaded
+ *   along other time elements (hour min etc) so as doing rounding properly is
+ *   hideously difficult, truncated instead, thanks Marco Turconi
+ *
+ * Revision 1.46  2004/04/03 08:22:22  csoutheren
+ * Remove pseudo-RTTI and replaced with real RTTI
+ *
+ * Revision 1.45  2003/08/14 02:07:11  dereksmithies
+ * Fix bug in AsString handling. Thanks Diego Tartara for pointing it out.
+ *
+ * Revision 1.44  2002/12/17 04:41:40  robertj
+ * Added ability to escape special characters.
+ *
+ * Revision 1.43  2002/12/10 05:22:16  robertj
+ * Fixed GNU warning
+ *
+ * Revision 1.42  2002/12/10 04:45:41  robertj
+ * Added support in PTime for ISO 8601 format.
+ *
+ * Revision 1.41  2002/02/15 03:56:09  yurik
+ * Warnings removed during compilation, patch courtesy of Jehan Bing, jehan@bravobrava.com
+ *
  * Revision 1.40  2001/10/16 07:44:06  robertj
  * Added AsString() function to PTimeInterval.
  *
@@ -180,7 +203,7 @@ PTimeInterval::PTimeInterval(const PString & str)
 
 PObject::Comparison PTimeInterval::Compare(const PObject & obj) const
 {
-  PAssert(obj.IsDescendant(PTimeInterval::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, PTimeInterval), PInvalidCast);
   const PTimeInterval & other = (const PTimeInterval &)obj;
   return milliseconds < other.milliseconds ? LessThan :
          milliseconds > other.milliseconds ? GreaterThan : EqualTo;
@@ -345,7 +368,7 @@ static time_t p_mktime(struct tm * t, int zone)
     theTime = 0;
   else if (zone != PTime::Local) {
     theTime += PTime::GetTimeZone()*60;  // convert to local time
-    if (theTime > zone*60)
+    if (theTime > (time_t) zone*60)
       theTime -= zone*60;           // and then to GMT
   }
   return theTime;
@@ -385,7 +408,7 @@ PTime::PTime(int second, int minute, int hour,
 
 PObject::Comparison PTime::Compare(const PObject & obj) const
 {
-  PAssert(obj.IsDescendant(PTime::Class()), PInvalidCast);
+  PAssert(PIsDescendant(&obj, PTime), PInvalidCast);
   const PTime & other = (const PTime &)obj;
   if (theTime < other.theTime)
     return LessThan;
@@ -402,16 +425,27 @@ PObject::Comparison PTime::Compare(const PObject & obj) const
 
 
 PString PTime::AsString(TimeFormat format, int zone) const
-{
-  if (format == RFC1123)
-    return AsString("wwwe, dd MMME yyyy hh:mm:ss z", zone);
+{ 
+  if (format >= NumTimeStrings)
+    return "Invalid format : " + AsString("yyyy-MM-dd T hh:mm:ss Z");
+
+  switch (format) {
+    case RFC1123 :
+      return AsString("wwwe, dd MMME yyyy hh:mm:ss z", zone);
+    case ShortISO8601 :
+      return AsString("yyyyMMddThhmmssZ");
+    case LongISO8601 :
+      return AsString("yyyy-MM-dd T hh:mm:ss Z");
+    default :
+      break;
+  }
 
   PString fmt, dsep;
 
   PString tsep = GetTimeSeparator();
   BOOL is12hour = GetTimeAMPM();
 
-  switch (format & 7) {
+  switch (format ) {
     case LongDateTime :
     case LongTime :
     case MediumDateTime :
@@ -439,7 +473,7 @@ PString PTime::AsString(TimeFormat format, int zone) const
       break;
   }
 
-  switch (format & 7) {
+  switch (format ) {
     case LongDateTime :
     case MediumDateTime :
     case ShortDateTime :
@@ -450,7 +484,7 @@ PString PTime::AsString(TimeFormat format, int zone) const
       break;
   }
 
-  switch (format & 7) {
+  switch (format ) {
     case LongDateTime :
     case LongDate :
       fmt += "wwww ";
@@ -604,15 +638,20 @@ PString PTime::AsString(const char * format, int zone) const
         break;
 
       case 'z' :
-        while (*++format == 'z')
-          ;
-        if (zone == 0)
-          str << "GMT";
+      case 'Z' :
+        if (zone == 0) {
+          if (*format == 'Z')
+            str << 'Z';
+          else
+            str << "GMT";
+        }
         else {
           str << (zone < 0 ? '-' : '+');
           zone = PABS(zone);
           str << setw(2) << (zone/60) << setw(2) << (zone%60);
         }
+        while (toupper(*++format) == 'z')
+          ;
         break;
 
       case 'u' :
@@ -620,13 +659,13 @@ PString PTime::AsString(const char * format, int zone) const
           repeatCount++;
         switch (repeatCount) {
           case 1 :
-            str << ((microseconds+50000)/100000);
+            str << (microseconds/100000);
             break;
           case 2 :
-            str << setw(2) << ((microseconds+5000)/10000);
+            str << setw(2) << (microseconds/10000);
             break;
           case 3 :
-            str << setw(3) << ((microseconds+500)/1000);
+            str << setw(3) << (microseconds/1000);
             break;
           default :
             str << setw(6) << microseconds;
@@ -634,6 +673,10 @@ PString PTime::AsString(const char * format, int zone) const
         }
         break;
 
+      case '\\' :
+        format++;
+        // Escaped character, put straight through to output string
+
       default :
         str << *format++;
     }
