@@ -26,6 +26,23 @@
  *   Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
  * $Log: vconvert.cxx,v $
+ * Revision 1.45.2.12  2007/06/18 19:23:27  dsandras
+ * Fixed previous commit.
+ *
+ * Revision 1.45.2.11  2007/06/18 18:23:52  dsandras
+ * Added patch from Elaine Xiong <elaine xiong sun com> to optimize
+ * color space conversions using MediaLib.
+ *
+ * Revision 1.45.2.10  2007/06/09 18:42:20  dsandras
+ * Backported patch from HEAD.
+ *
+ * Revision 1.45.2.9  2006/12/30 21:48:54  dsandras
+ * Backported patch from HEAD that fixes Ekiga bug #333311.
+ *
+ * Revision 1.45.2.8  2006/11/01 17:48:13  dsandras
+ * Added patch from Brian Lu <brian lu sun com> to fix compilation on
+ * OpenSolaris.
+ *
  * Revision 1.45.2.7  2006/04/30 21:28:38  dsandras
  * Backport from HEAD.
  *
@@ -247,10 +264,14 @@
 
 #include <ptlib/vconvert.h>
 
-#ifdef __GNUC__
+#if  defined(__GNUC__) || defined(__sun) 
 #include "tinyjpeg.h"
 #endif
 
+#ifdef P_MEDIALIB
+#include <mlib.h>
+#endif
+
 static PColourConverterRegistration * RegisteredColourConvertersListHead = NULL;
 
 PSYNONYM_COLOUR_CONVERTER(SBGGR8, SBGGR8);
@@ -259,6 +280,7 @@ PSYNONYM_COLOUR_CONVERTER(RGB24,  RGB24);
 PSYNONYM_COLOUR_CONVERTER(BGR24,  BGR24);
 PSYNONYM_COLOUR_CONVERTER(RGB32,  RGB32);
 PSYNONYM_COLOUR_CONVERTER(BGR32,  BGR32);
+PSYNONYM_COLOUR_CONVERTER(UYVY422,UYVY422);
 PSYNONYM_COLOUR_CONVERTER(YUV411P,YUV411P);
 PSYNONYM_COLOUR_CONVERTER(YUV420P,IYUV);
 PSYNONYM_COLOUR_CONVERTER(IYUV,   YUV420P);
@@ -337,6 +359,10 @@ class PStandardColourConverter : public PColourConverter
       unsigned srcIncrement,
       unsigned dstIncrement
     ) const;
+    void ResizeUYVY422(
+      const BYTE *src_uyvy,
+      BYTE *dst_uyvy
+    ) const;
     void ResizeYUV422(
       const BYTE * src,
       BYTE * dest
@@ -365,7 +391,7 @@ class PStandardColourConverter : public PColourConverter
       const BYTE *yuy2,
       BYTE *yuv420p
     );
-#ifdef __GNUC__
+#if defined (__GNUC__) || defined (__sun)
     bool MJPEGtoXXX(
       const BYTE *mjpeg,
 	    BYTE *output_data,
@@ -1572,6 +1598,27 @@ BOOL PStandardColourConverter::YUV420PtoRGB(const BYTE * srcFrameBuffer,
   const BYTE    *uplane    = yplane+nbytes;                // 1 byte U for a block of 4 pixels
   const BYTE    *vplane    = uplane+(nbytes/4);            // 1 byte V for a block of 4 pixels
 
+#ifdef P_MEDIALIB
+  const BYTE *y0;
+  const BYTE *y1;
+  const BYTE *cb;
+  const BYTE *cr;
+  unsigned int   x,p;
+
+  for(int i = 0; i < srcFrameHeight; i += 2) {
+    p = i*srcFrameWidth;
+    x = p/4;
+    y0 = yplane + p;
+    y1 = y0 + srcFrameWidth;
+    cb = uplane + x;
+    cr = vplane + x;
+    mlib_VideoColorJFIFYCC2RGB420_Nearest(dstFrameBuffer,
+                                          dstFrameBuffer+3*dstFrameWidth, 
+                                          y0, y1, cb, cr,
+                                          srcFrameWidth);
+    dstFrameBuffer += 6*dstFrameWidth;
+  }
+#else
   unsigned int   pixpos[4] = {0, 1, srcFrameWidth, srcFrameWidth + 1};
   unsigned int   originalPixpos[4] = {0, 1, srcFrameWidth, srcFrameWidth + 1};
 
@@ -1637,6 +1684,7 @@ BOOL PStandardColourConverter::YUV420PtoRGB(const BYTE * srcFrameBuffer,
 
   if (bytesReturned != NULL)
     *bytesReturned = dstFrameBytes;
+#endif  
 
   return TRUE;
 }
@@ -2010,6 +2058,119 @@ PSTANDARD_COLOUR_CONVERTER(YUV411P,YUV420P)
   return TRUE;
 }
 
+
+/*
+ * Format UYVY or UYVY422(non planar) 4x4
+ *
+ * off: 0  U00 Y00 V01 Y00 U02 Y01 V03 Y01
+ * off: 8  U10 Y10 V11 Y10 U12 Y11 V13 Y11
+ * off:16  U20 Y20 V21 Y20 U22 Y21 V23 Y21
+ * off:24  U30 Y30 V31 Y30 U32 Y31 V33 Y31
+ * length:32 bytes
+ */
+PSTANDARD_COLOUR_CONVERTER(UYVY422, UYVY422)
+{
+  if (bytesReturned != NULL)
+    *bytesReturned = dstFrameBytes;
+  
+  if (srcFrameBuffer == dstFrameBuffer) {
+	if (srcFrameWidth == dstFrameWidth && srcFrameHeight == dstFrameHeight) 
+		return TRUE;
+	else if(srcFrameWidth < dstFrameWidth || srcFrameHeight < dstFrameHeight)
+		return FALSE;
+  }
+
+  if ((srcFrameWidth == dstFrameWidth) && (srcFrameHeight == dstFrameHeight)) 
+    memcpy(dstFrameBuffer,srcFrameBuffer,srcFrameWidth*srcFrameHeight*2);
+  else
+    ResizeUYVY422(srcFrameBuffer, dstFrameBuffer);
+
+  return TRUE;
+}
+
+/*
+ * Format UYVY (or UYVY422) non planar (4x4)
+ *
+ * off: 0  U00 Y00 V01 Y00 U02 Y01 V03 Y01
+ * off: 8  U10 Y10 V11 Y10 U12 Y11 V13 Y11
+ * off:16  U20 Y20 V21 Y20 U22 Y21 V23 Y21
+ * off:24  U30 Y30 V31 Y30 U32 Y31 V33 Y31
+ * length:32 bytes
+ *
+ * NOTE: This algorithm works only if the width and the height is pair.
+ */
+void PStandardColourConverter::ResizeUYVY422(const BYTE *src_uyvy, BYTE *dst_uyvy) const
+{
+  const BYTE *s;
+  BYTE *d;
+  unsigned int x, h;  
+  unsigned int npixels = dstFrameWidth * dstFrameHeight;
+
+  s = src_uyvy;
+  d = dst_uyvy;
+
+  if ( (srcFrameWidth * srcFrameHeight) < npixels ) {
+
+     // dest is bigger than the source. No subsampling.
+     // Place the src in the middle of the destination.
+     unsigned int yOffset = (dstFrameHeight - srcFrameHeight)/2;
+     unsigned int xOffset = (dstFrameWidth - srcFrameWidth)/2;
+
+     /* Top border */
+     for (h=0; h<yOffset; h++)
+      {
+	for (x=0; x<dstFrameWidth/2; x++)
+	 {
+	   *d++ = BLACK_U;
+	   *d++ = BLACK_Y;
+	   *d++ = BLACK_V;
+	   *d++ = BLACK_Y;
+	 }
+      }
+
+     for (h=0; h<srcFrameHeight; h+=2)
+      {
+        /* Left border */
+       	for (x=0; x<xOffset/2; x++)
+	 {
+	   *d++ = BLACK_U;
+	   *d++ = BLACK_Y;
+	   *d++ = BLACK_V;
+	   *d++ = BLACK_Y;
+	 }
+
+        /* Copy the first line keeping all information */
+	memcpy(d, s, srcFrameWidth*2);
+	d += srcFrameWidth*2;
+        /* Right and Left border */
+       	for (x=0; x<xOffset/2; x++)
+	 {
+	   *d++ = BLACK_U;
+	   *d++ = BLACK_Y;
+	   *d++ = BLACK_V;
+	   *d++ = BLACK_Y;
+	 }
+      }
+     for (h=0; h<yOffset; h++)
+      {
+	for (x=0; x<dstFrameWidth/2; x++)
+	 {
+	   *d++ = BLACK_U;
+	   *d++ = BLACK_Y;
+	   *d++ = BLACK_V;
+	   *d++ = BLACK_Y;
+	 }
+      }
+
+  } else {
+
+     /* FIXME */
+
+   }
+
+}
+
+
 /*
  * Format UYVY or UYVY422(non planar) 4x4
  *
@@ -2285,7 +2446,7 @@ PSTANDARD_COLOUR_CONVERTER(UYV444,YUV420P)
   return TRUE;
 }
 
-#ifdef __GNUC__
+#if  defined (__GNUC__) || defined (__sun)
 #ifndef P_MACOSX
 /*
  * Convert a MJPEG Buffer to one plane pixel format (RGB24, BGR24, GRAY)
@@ -2432,7 +2593,8 @@ bool PStandardColourConverter::MJPEGtoYUV420P(const BYTE *mjpeg,
 
   } else {
      /* Very not efficient */
-     BYTE *intermed = intermediateFrameStore.GetPointer(srcFrameBytes);
+     unsigned int bytes = srcFrameWidth * srcFrameHeight * 3 / 2;
+     BYTE *intermed = intermediateFrameStore.GetPointer(bytes);
      MJPEGtoYUV420PSameSize(mjpeg, intermed);
      ResizeYUV420P(intermed, yuv420p);
   }
