@@ -27,8 +27,20 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: ptlib.cxx,v $
- * Revision 1.73.2.1  2005/02/04 05:19:11  csoutheren
- * Backported patches from Atlas-devel
+ * Revision 1.79  2005/11/30 12:47:42  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.78  2005/09/24 09:11:42  dominance
+ * use unix-style slashes to not confuse mingw on win32
+ *
+ * Revision 1.77  2005/09/23 15:30:46  dominance
+ * more progress to make mingw compile nicely. Thanks goes to Julien Puydt for pointing out to me how to do it properly. ;)
+ *
+ * Revision 1.76  2005/08/05 20:41:42  csoutheren
+ * Added unix support for scattered read/write
+ *
+ * Revision 1.75  2005/08/05 19:42:09  csoutheren
+ * Added support for scattered read/write
  *
  * Revision 1.74  2004/12/08 00:49:37  csoutheren
  * Fixed weird problem with not returning correct filetype when filename has multiple
@@ -279,8 +291,10 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <share.h>
-#include <sys\stat.h>
+#include <sys/stat.h>
+#ifdef _MSC_VER
 #include <crtdbg.h>
+#endif
 
 #if !P_USE_INLINES
 #include <ptlib/osutil.inl>
@@ -500,7 +514,6 @@ BOOL PChannel::Close()
   return FALSE;
 }
 
-
 ///////////////////////////////////////////////////////////////////////////////
 // Directories
 
@@ -849,7 +862,7 @@ BOOL PFile::GetInfo(const PFilePath & name, PFileInfo & info)
     info.created = FileTimeToTime(FInfo.ftCreationTime);
     info.modified = FileTimeToTime(FInfo.ftLastWriteTime);
     info.accessed = FileTimeToTime(FInfo.ftLastAccessTime);
-    info.size = (__int64(FInfo.nFileSizeHigh)<<32)+__int64(FInfo.nFileSizeLow);		
+    info.size = (__int64(FInfo.nFileSizeHigh)<<32)+__int64(FInfo.nFileSizeLow);
     
     info.permissions = PFileInfo::UserRead|PFileInfo::GroupRead|PFileInfo::WorldRead;
     
