@@ -27,6 +27,25 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: config.cxx,v $
+ * Revision 1.36  2004/04/03 23:55:59  csoutheren
+ * Added fix for PConfig environment variables under Linux
+ *   Thanks to Michal Zygmuntowicz
+ *
+ * Revision 1.35  2003/03/17 08:10:59  robertj
+ * Fixed bug with parsing lines with no equal sign
+ *
+ * Revision 1.34  2003/01/30 23:46:05  dereks
+ * Fix compile error on gcc 3.2
+ *
+ * Revision 1.33  2003/01/26 03:57:12  robertj
+ * Fixed problem with last change so can still operate if do not have write
+ *   access to the directory config file is in.
+ * Improved error reporting.
+ *
+ * Revision 1.32  2003/01/24 12:12:20  robertj
+ * Changed so that a crash in some other thread can no longer cause the
+ *   config file to be truncated to zero bytes.
+ *
  * Revision 1.31  2001/09/14 07:36:27  robertj
  * Fixed bug where fails to read last line of config file if not ended with '\n'.
  *
@@ -286,13 +305,16 @@ BOOL PXConfig::WriteToFile(const PFilePath & filename)
                                    PFileInfo::UserExecute |
                                    PFileInfo::UserWrite |
                                    PFileInfo::UserRead)) {
-    PProcess::PXShowSystemWarning(2000, "Cannot create PWLIB config dir");
+    PProcess::PXShowSystemWarning(2000, "Cannot create PWLIB config directory");
     return FALSE;
   }
 
   PTextFile file;
-  if (!file.Open(filename, PFile::WriteOnly)) {
-    PProcess::PXShowSystemWarning(2001, "Cannot create PWLIB config file");
+  if (!file.Open(filename + ".new", PFile::WriteOnly))
+    file.Open(filename, PFile::WriteOnly);
+
+  if (!file.IsOpen()) {
+    PProcess::PXShowSystemWarning(2001, "Cannot create PWLIB config file: " + file.GetErrorText());
     return FALSE;
   }
 
@@ -311,12 +333,21 @@ BOOL PXConfig::WriteToFile(const PFilePath & filename)
 
   file.flush();
   file.SetLength(file.GetPosition());
+  file.Close();
 
+  if (file.GetFilePath() != filename) {
+    if (!file.Rename(file.GetFilePath(), filename.GetFileName(), TRUE)) {
+      PProcess::PXShowSystemWarning(2001, "Cannot rename config file: " + file.GetErrorText());
+      return FALSE;
+    }
+  }
+
+  PTRACE(4, "PWLib\tSaved config file: " << filename);
   return TRUE;
 }
 
 
-BOOL PXConfig::ReadFromFile (const PFilePath & filename)
+BOOL PXConfig::ReadFromFile(const PFilePath & filename)
 {
   PINDEX len;
 
@@ -351,7 +382,7 @@ BOOL PXConfig::ReadFromFile (const PFilePath & filename)
           }
         } else if (currentSection != NULL) {
           PINDEX equals = line.Find('=');
-          if (equals > 0) {
+          if (equals > 0 && equals != P_MAX_INDEX) {
             PString keyStr = line.Left(equals).Trim();
             PString valStr = line.Right(len - equals - 1).Trim();
 
@@ -545,13 +576,15 @@ void PConfig::Construct(Source src,
                         const PString & appname,
                         const PString & /*manuf*/)
 {
+  // handle cnvironment configs differently
+  if (src == PConfig::Environment)  {
+    config = configDict->GetEnvironmentInstance();
+    return;
+  }
+  
   PString name;
   PFilePath filename, readFilename;
-
-  // handle cnvironment configs differently
-  if (src == PConfig::Environment) 
-    config = configDict->GetEnvironmentInstance();
-
+  
   // look up file name to read, and write
   if (src == PConfig::System)
     LocateFile(SYS_CONFIG_NAME, readFilename, filename);
