@@ -27,6 +27,98 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: svcproc.cxx,v $
+ * Revision 1.89  2005/12/15 21:14:34  dsandras
+ * Fixed from Alexander Larsson <alexl _AT__ redhat.com> for gcc 4.1 compilation. Thanks!
+ *
+ * Revision 1.88  2005/11/30 12:47:42  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.87  2005/11/25 00:22:02  csoutheren
+ * Applied patch #1266244 from Nick Shmyrev
+ * Cleanup PSystemLog from PTrace in PService exit
+ *
+ * Revision 1.86  2004/06/01 07:42:19  csoutheren
+ * Restored memory allocation checking
+ * Added configure flag to enable, thanks to Derek Smithies
+ *
+ * Revision 1.85  2004/04/03 08:57:41  csoutheren
+ * Replaced pseudo-RTTI with real RTTI
+ *
+ * Revision 1.84  2004/02/22 03:31:50  ykiryanov
+ * Added current thread id routine to BeOS code
+ *
+ * Revision 1.83  2003/09/17 09:02:14  csoutheren
+ * Removed memory leak detection code
+ *
+ * Revision 1.82  2003/01/14 04:43:21  robertj
+ * Improved output on error in getting service status.
+ *
+ * Revision 1.81  2002/12/12 00:54:29  robertj
+ * Fixed issue with setting max file handles causing a log file to be created
+ *   as root (real uid) rather than as the user determined by the -u arg.
+ * Utilised new PProcess functions for setting user and group.
+ *
+ * Revision 1.80  2002/12/02 08:27:43  robertj
+ * Fixed incorrectly set #if statement from RTEMS patch.
+ *
+ * Revision 1.79  2002/12/02 03:57:18  robertj
+ * More RTEMS support patches, thank you Vladimir Nesic.
+ *
+ * Revision 1.78  2002/11/02 00:32:21  robertj
+ * Further fixes to VxWorks (Tornado) port, thanks Andreas Sikkema.
+ *
+ * Revision 1.77  2002/10/30 03:14:15  robertj
+ * Added tab character between time and thread in system log (for db import).
+ *
+ * Revision 1.76  2002/10/23 04:26:27  robertj
+ * Fixed extra debug levels in system log dump.
+ *
+ * Revision 1.75  2002/10/17 13:44:27  robertj
+ * Port to RTEMS, thanks Vladimir Nesic.
+ *
+ * Revision 1.74  2002/10/17 12:57:24  robertj
+ * Added ability to increase maximum file handles on a process.
+ *
+ * Revision 1.73  2002/10/10 04:43:44  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
+ * Revision 1.72  2002/10/03 01:27:51  robertj
+ * Added fail safe _exit() as it appears raise(SIGQUIT) does not always dump
+ *   core and exit the app causing endless SEGV's to appear the log file!
+ *
+ * Revision 1.71  2002/10/01 06:03:44  robertj
+ * Attempt to prevent recursive signals for SEGV etc
+ *
+ * Revision 1.70  2002/09/06 13:29:06  robertj
+ * Added missing help line for status command.
+ *
+ * Revision 1.69  2002/07/15 06:39:23  craigs
+ * Added function to allow raising of per-process file handle limit
+ *
+ * Revision 1.68  2002/06/13 08:50:11  rogerh
+ * GCC 3.1 uses slightly different #includes
+ *
+ * Revision 1.67  2002/06/05 12:29:16  craigs
+ * Changes for gcc 3.1
+ *
+ * Revision 1.66  2002/04/17 03:31:48  robertj
+ * Renamed system log file member variable to be common with Windows version.
+ *
+ * Revision 1.65  2002/03/18 08:03:30  robertj
+ * Fixed hex output of thread pid.
+ * Added ability to have -tk option wich attempt to terminate a daemon and
+ *   if no response in 10 seconds kills it.
+ * Fixed leaving .pid file behind when using -k option.
+ *
+ * Revision 1.64  2002/02/19 07:12:36  rogerh
+ * Mac Carbon fix
+ *
+ * Revision 1.63  2001/12/16 23:40:16  robertj
+ * Fixed system log so does not crash if current thread not created by PWLib.
+ *
+ * Revision 1.62  2001/12/09 23:45:20  craigs
+ * Set debugMode flag when in .... debug mode!
+ *
  * Revision 1.61  2001/09/20 05:35:47  robertj
  * Fixed crash (race condition) if shutdown service via signal and exit main.
  *
@@ -167,11 +259,28 @@
 #pragma implementation "svcproc.h"
 #include <ptlib/svcproc.h>
 
+#ifdef P_VXWORKS
+#include <logLib.h>
+#define LOG_EMERG      0
+#define LOG_ALERT      1
+#define LOG_CRIT      2
+#define LOG_ERR        3
+#define LOG_WARNING      4
+#define  LOG_NOTICE      5
+#define LOG_INFO      6
+#define LOG_DEBUG      7
+#else
 #include <syslog.h>
-#include <stdarg.h>
-#include <fstream.h>
 #include <pwd.h>
 #include <grp.h>
+#endif
+
+#include <stdarg.h>
+#if (__GNUC__ >= 3)
+#include <fstream>
+#else
+#include <fstream.h>
+#endif
 #include <signal.h>
 
 #include "uerror.h"
@@ -182,9 +291,11 @@
 
 #define new PNEW
 
+extern void PXSignalHandler(int);
 
-#define	MAX_LOG_LINE_LEN	1024
+#define  MAX_LOG_LINE_LEN  1024
 
+#ifndef P_VXWORKS
 static int PwlibLogToUnixLog[PSystemLog::NumLogLevels] = {
   LOG_CRIT,    // LogFatal,   
   LOG_ERR,     // LogError,   
@@ -192,8 +303,12 @@ static int PwlibLogToUnixLog[PSystemLog::NumLogLevels] = {
   LOG_INFO,    // LogInfo,    
   LOG_DEBUG,   // LogDebug
   LOG_DEBUG,
+  LOG_DEBUG,
+  LOG_DEBUG,
+  LOG_DEBUG,
   LOG_DEBUG
 };
+#endif // !P_VXWORKS
 
 static const char * const PLevelName[PSystemLog::NumLogLevels+1] = {
   "Message",
@@ -203,7 +318,10 @@ static const char * const PLevelName[PSystemLog::NumLogLevels+1] = {
   "Info",
   "Debug",
   "Debug2",
-  "Debug3"
+  "Debug3",
+  "Debug4",
+  "Debug5",
+  "Debug6",
 };
 
 #ifdef P_MAC_MPTHREADS
@@ -234,9 +352,15 @@ static pthread_mutex_t logMutex = PTHREAD_MUTEX_INITIALIZER;
 
 void PSystemLog::Output(Level level, const char * cmsg)
 {
-  PString systemLogFile = PServiceProcess::Current().systemLogFile;
-  if (systemLogFile.IsEmpty())
+  PString systemLogFileName = PServiceProcess::Current().systemLogFileName;
+  if (systemLogFileName.IsEmpty()) {
+#ifdef P_VXWORKS
+    printf("%s\n",cmsg);
+    logMsg((char *)(const char)cmsg,0,0,0,0,0,0);
+#else
     syslog(PwlibLogToUnixLog[level], "%s", cmsg);
+#endif
+  }
   else {
 #ifdef P_PTHREADS
     pthread_mutex_lock(&logMutex);
@@ -247,18 +371,37 @@ void PSystemLog::Output(Level level, const char * cmsg)
 #endif
 
     ostream * out;
-    if (systemLogFile == "-")
+    if (systemLogFileName == "-")
       out = &cerr;
     else
-      out = new ofstream(systemLogFile, ios::app);
+      out = new ofstream(systemLogFileName, ios::app);
 
     PTime now;
-    *out << now.AsString("yyyy/MM/dd hh:mm:ss.uuu ");
-    PString threadName = PThread::Current()->GetThreadName();
-    if (threadName.GetLength() <= 23)
-      *out << setw(23) << threadName;
-    else
-      *out << threadName.Left(10) << "..." << threadName.Right(10);
+    *out << now.AsString("yyyy/MM/dd hh:mm:ss.uuu\t");
+
+    PThread * thread = PThread::Current();
+    if (thread == NULL) {
+#ifdef P_MAC_MPTHREADS
+      unsigned tid = (unsigned)MPCurrentTaskID();
+#elif defined(P_VXWORKS)
+      unsigned tid = ::taskIdSelf();
+#elif defined(BE_THREADS)
+      thread_id tid = ::find_thread(NULL);
+#else
+      unsigned tid = (unsigned) pthread_self();
+#endif
+      *out << "ThreadID=0x"
+           << setfill('0') << ::hex
+           << setw(8) << tid
+           << setfill(' ') << ::dec;
+    } else {
+      PString threadName = thread->GetThreadName();
+      if (threadName.GetLength() <= 23)
+        *out << setw(23) << threadName;
+      else
+        *out << threadName.Left(10) << "..." << threadName.Right(10);
+    }
+
     *out << '\t'
          << PLevelName[level+1]
          << '\t'
@@ -325,34 +468,66 @@ PServiceProcess::PServiceProcess(const char * manuf,
 
 PServiceProcess::~PServiceProcess()
 {
+  PSetErrorStream(NULL);
+  PTrace::SetStream(NULL);
+  PTrace::ClearOptions(PTrace::SystemLogStream);
+
   if (!pidFileToRemove)
     PFile::Remove(pidFileToRemove);
 
+#ifndef P_VXWORKS
   // close the system log
-  if (systemLogFile.IsEmpty())
+  if (systemLogFileName.IsEmpty())
     closelog();
+#endif // !P_VXWORKS
 }
 
 
 PServiceProcess & PServiceProcess::Current()
 {
   PProcess & process = PProcess::Current();
-  PAssert(process.IsDescendant(PServiceProcess::Class()), "Not a service process!");
+  PAssert(PIsDescendant(&process, PServiceProcess), "Not a service process!");
   return (PServiceProcess &)process;
 }
 
+
+#ifndef P_VXWORKS
+static int KillProcess(int pid, int sig)
+{
+  if (kill(pid, sig) != 0)
+    return -1;
+
+  cout << "Sent SIG";
+  if (sig == SIGTERM)
+    cout << "TERM";
+  else
+    cout << "KILL";
+  cout << " to daemon at pid " << pid << ' ' << flush;
+
+  for (PINDEX retry = 1; retry <= 10; retry++) {
+    PThread::Sleep(1000);
+    if (kill(pid, 0) != 0) {
+      cout << "\nDaemon stopped." << endl;
+      return 0;
+    }
+    cout << '.' << flush;
+  }
+  cout << "\nDaemon has not stopped." << endl;
+
+  return 1;
+}
+#endif // !P_VXWORKS
+
+
 void PServiceProcess::_PXShowSystemWarning(PINDEX code, const PString & str)
 {
-  PSYSTEMLOG(Warning, "PWLib/Unix error #"
-                      << code
-                      << "-"
-                      << str
-                      << endl);
+  PSYSTEMLOG(Warning, "PWLib\t" << GetOSClass() << " error #" << code << '-' << str);
 }
 
 
 int PServiceProcess::InitialiseService()
 {
+#ifndef P_VXWORKS
 #if PMEMORY_CHECK
   PMemoryHeap::SetIgnoreAllocations(TRUE);
 #endif
@@ -364,6 +539,7 @@ int PServiceProcess::InitialiseService()
 #if PMEMORY_CHECK
   PMemoryHeap::SetIgnoreAllocations(FALSE);
 #endif
+  debugMode = FALSE;
 
   // parse arguments so we can grab what we want
   PArgList & args = GetArguments();
@@ -374,6 +550,7 @@ int PServiceProcess::InitialiseService()
              "h-help."
              "x-execute."
              "p-pid-file:"
+       "H-handlemax:"
              "i-ini-file:"
              "k-kill."
              "t-terminate."
@@ -406,46 +583,56 @@ int PServiceProcess::InitialiseService()
     pidfilename = PDirectory(pidfilename) + PProcess::Current().GetFile().GetFileName() + ".pid";
 
   if (args.HasOption('k') || args.HasOption('t') || args.HasOption('s')) {
-    ifstream pidfile(pidfilename);
-    if (!pidfile.is_open()) {
-      cout << "Could not open pid file: \"" << pidfilename << "\""
-              " - " << strerror(errno) << endl;
-      return 1;
-    }
-
     pid_t pid;
-    pidfile >> pid;
-    if (pid == 0) {
-      cout << "Illegal format pid file \"" << pidfilename << '"' << endl;
-      return 1;
+
+    {
+      ifstream pidfile(pidfilename);
+      if (!pidfile.is_open()) {
+        cout << "Could not open pid file: \"" << pidfilename << "\""
+                " - " << strerror(errno) << endl;
+        return 1;
+      }
+
+      pidfile >> pid;
+      if (pid == 0) {
+        cout << "Illegal format pid file \"" << pidfilename << '"' << endl;
+        return 1;
+      }
     }
 
     if (args.HasOption('s')) {
-      cout << "Process at " << pid << " is ";
-      if (kill(pid, 0) != 0)
-        cout << "not ";
-      cout << "running." << endl;
+      cout << "Process at " << pid << ' ';
+      if (kill(pid, 0) == 0)
+        cout << "is running.";
+      else if (errno == ESRCH)
+        cout << "does not exist.";
+      else
+        cout << " status could not be determined, error: " << strerror(errno);
+      cout << endl;
       return 0;
     }
 
     int sig = args.HasOption('t') ? SIGTERM : SIGKILL;
-    if (kill(pid, sig) == 0) {
-      cout << "Sent SIG";
-      if (sig == SIGTERM)
-        cout << "TERM";
-      else
-        cout << "KILL";
-      cout << " to daemon at pid " << pid << ' ' << flush;
-      for (PINDEX retry = 1; retry <= 5; retry++) {
-        Sleep(1000);
-        if (kill(pid, 0) != 0) {
-          cout << "\nDaemon stopped." << endl;
-          return 0;
+    switch (KillProcess(pid, sig)) {
+      case -1 :
+        break;
+      case 0 :
+        PFile::Remove(pidfilename);
+        return 0;
+      case 1 :
+        if (args.HasOption('t') && args.HasOption('k')) {
+          switch (KillProcess(pid, SIGKILL)) {
+            case -1 :
+              break;
+            case 0 :
+              PFile::Remove(pidfilename);
+              return 0;
+            case 1 :
+              return 2;
+          }
         }
-        cout << '.' << flush;
-      }
-      cout << "\nDaemon has not stopped." << endl;
-      return 2;
+        else
+          return 2;
     }
 
     cout << "Could not stop process " << pid <<
@@ -453,6 +640,24 @@ int PServiceProcess::InitialiseService()
     return 1;
   }
 
+  // Set the gid we are running under
+  if (args.HasOption('g')) {
+    PString gidstr = args.GetOptionString('g');
+    if (!SetGroupName(gidstr)) {
+      cout << "Could not set GID to \"" << gidstr << "\" - " << strerror(errno) << endl;
+      return 1;
+    }
+  }
+
+  // Set the uid we are running under
+  if (args.HasOption('u')) {
+    PString uidstr = args.GetOptionString('u');
+    if (!SetUserName(uidstr)) {
+      cout << "Could not set UID to \"" << uidstr << "\" - " << strerror(errno) << endl;
+      return 1;
+    }
+  }
+
   BOOL helpAndExit = FALSE;
 
   // if displaying help, then do it
@@ -464,24 +669,26 @@ int PServiceProcess::InitialiseService()
   }
 
   // set flag for console messages
-  if (args.HasOption('c'))
-    systemLogFile = '-';
+  if (args.HasOption('c')) {
+    systemLogFileName = '-';
+    debugMode = TRUE;
+  }
 
   if (args.HasOption('l')) {
-    systemLogFile = args.GetOptionString('l');
-    if (systemLogFile.IsEmpty()) {
+    systemLogFileName = args.GetOptionString('l');
+    if (systemLogFileName.IsEmpty()) {
       cout << "error: must specify file name for -l" << endl;
       helpAndExit = TRUE;
     }
-    else if (PDirectory::Exists(systemLogFile))
-      systemLogFile = PDirectory(systemLogFile) + PProcess::Current().GetFile().GetFileName() + ".log";
+    else if (PDirectory::Exists(systemLogFileName))
+      systemLogFileName = PDirectory(systemLogFileName) + PProcess::Current().GetFile().GetFileName() + ".log";
   }
 
   if (helpAndExit) {
     cout << "usage: [-c] -v|-d|-h|-x\n"
             "  -h --help           output this help message and exit\n"
             "  -v --version        display version information and exit\n"
-#ifndef BE_THREADS
+#if !defined(BE_THREADS) && !defined(P_RTEMS)
             "  -d --daemon         run as a daemon\n"
 #endif
             "  -u --uid uid        set user id to run as\n"
@@ -489,11 +696,13 @@ int PServiceProcess::InitialiseService()
             "  -p --pid-file       name or directory for pid file\n"
             "  -t --terminate      orderly terminate process in pid file\n"
             "  -k --kill           preemptively kill process in pid file\n"
+            "  -s --status         check to see if daemon is running\n"
             "  -c --console        output messages to stdout rather than syslog\n"
             "  -l --log-file file  output messages to file or directory instead of syslog\n"
             "  -x --execute        execute as a normal program\n"
             "  -i --ini-file       set the ini file to use, may be explicit file or\n"
             "                      a ':' separated set of directories to search.\n"
+            "  -H --handlemax n    set maximum number of file handles (set before uid/gid)\n"
 #ifdef P_LINUX
             "  -C --core-size      set the maximum core file size\n"
 #endif
@@ -501,53 +710,29 @@ int PServiceProcess::InitialiseService()
     return 0;
   }
 
-  if (args.HasOption('i'))
-    SetConfigurationPath(args.GetOptionString('i'));
-
-  // Set the gid we are running under
-  if (args.HasOption('g')) {
-    PString gidstr = args.GetOptionString('g');
-    if (gidstr.IsEmpty())
-      gidstr = "nobody";
-    int gid;
-    if (strspn(gidstr, "0123456789") == gidstr.GetLength())
-      gid = gidstr.AsInteger();
-    else {
-      struct group * gr = getgrnam(gidstr);
-      if (gr == NULL) {
-        cout << "Could not find group \"" << gidstr << '"' << endl;
-        return 1;
-      }
-      gid = gr->gr_gid;
-    }
-    if (setgid(gid) != 0) {
-      cout << "Could not set GID to \"" << gidstr << "\" (" << gid << ")"
+  // open the system logger for this program
+  if (systemLogFileName.IsEmpty())
+    openlog((char *)(const char *)GetName(), LOG_PID, LOG_DAEMON);
+  else if (systemLogFileName == "-")
+    cout << "All output for " << GetName() << " is to console." << endl;
+  else {
+    ofstream logfile(systemLogFileName, ios::app);
+    if (!logfile.is_open()) {
+      cout << "Could not open log file \"" << systemLogFileName << "\""
               " - " << strerror(errno) << endl;
       return 1;
     }
   }
+  PSYSTEMLOG(StdError, "Starting service process \"" << GetName() << "\" v" << GetVersion(TRUE));
 
-  // Set the uid we are running under
-  if (args.HasOption('u')) {
-    PString uidstr = args.GetOptionString('u');
-    if (uidstr.IsEmpty())
-      uidstr = "nobody";
-    int uid;
-    if (strspn(uidstr, "0123456789") == uidstr.GetLength())
-      uid = uidstr.AsInteger();
-    else {
-      struct passwd * pw = getpwnam(uidstr);
-      if (pw == NULL) {
-        cout << "Could not find user \"" << uidstr << '"' << endl;
-        return 1;
-      }
-      uid = pw->pw_uid;
-    }
-    if (setuid(uid) != 0) {
-      cout << "Could not set UID to \"" << uidstr << "\" (" << uid << ")"
-              " - " << strerror(errno) << endl;
-      return 1;
-    }
+  if (args.HasOption('i'))
+    SetConfigurationPath(args.GetOptionString('i'));
+
+  if (args.HasOption('H')) {
+    int uid = geteuid();
+    seteuid(getuid()); // Switch back to starting uid for next call
+    SetMaxHandles(args.GetOptionString('H').AsInteger());
+    seteuid(uid);
   }
 
   // set the core file size
@@ -557,6 +742,8 @@ int PServiceProcess::InitialiseService()
     if (getrlimit(RLIMIT_CORE, &rlim) != 0) 
       cout << "Could not get current core file size : error = " << errno << endl;
     else {
+      int uid = geteuid();
+      seteuid(getuid()); // Switch back to starting uid for next call
       int v = args.GetOptionString('C').AsInteger();
       rlim.rlim_cur = v;
       if (setrlimit(RLIMIT_CORE, &rlim) != 0) 
@@ -565,25 +752,12 @@ int PServiceProcess::InitialiseService()
         getrlimit(RLIMIT_CORE, &rlim);
         cout << "Core file size set to " << rlim.rlim_cur << "/" << rlim.rlim_max << endl;
       }
+      seteuid(uid);
     }
 #endif
   }
 
-  // open the system logger for this program
-  if (systemLogFile.IsEmpty())
-    openlog((char *)(const char *)GetName(), LOG_PID, LOG_DAEMON);
-  else if (systemLogFile == "-")
-    cout << "All output for " << GetName() << " is to console." << endl;
-  else {
-    ofstream logfile(systemLogFile, ios::app);
-    if (!logfile.is_open()) {
-      cout << "Could not open log file \"" << systemLogFile << "\""
-              " - " << strerror(errno) << endl;
-      return 1;
-    }
-  }
-
-#ifndef BE_THREADS
+#if !defined(BE_THREADS) && !defined(P_RTEMS)
   if (!args.HasOption('d'))
     return -1;
 
@@ -645,16 +819,14 @@ int PServiceProcess::InitialiseService()
   // Also if in background, don't want to get blocked on input from stdin
   ::close(STDIN_FILENO);
 
-#endif // BE_THREADS
-
+#endif // !BE_THREADS && !P_RTEMS
+#endif // !P_VXWORKS
   return -1;
 }
 
 int PServiceProcess::_main(void *)
 {
   if ((terminationValue = InitialiseService()) < 0) {
-    PSYSTEMLOG(Warning, "Starting service process \"" << GetName() << "\" v" << GetVersion(TRUE));
-
     // Make sure housekeeping thread is going so signals are handled.
     SignalTimerChange();
 
@@ -706,9 +878,11 @@ void PServiceProcess::Terminate()
   // Do the services stop code
   OnStop();
 
+#ifndef P_VXWORKS
   // close the system log
-  if (systemLogFile.IsEmpty())
+  if (systemLogFileName.IsEmpty())
     closelog();
+#endif // !P_VXWORKS
 
   // Now end the program
   exit(terminationValue);
@@ -745,14 +919,24 @@ void PServiceProcess::PXOnAsyncSignal(int sig)
       return;
   }
 
+  signal(SIGSEGV, SIG_DFL);
+  signal(SIGFPE, SIG_DFL);
+  signal(SIGBUS, SIG_DFL);
+
   static BOOL inHandler = FALSE;
-  if (inHandler)
+  if (inHandler) {
     raise(SIGQUIT); // Dump core
+    _exit(-1); // Fail safe if raise() didn't dump core and exit
+  }
 
   inHandler = TRUE;
 
-#if P_MAC_MPTHREADS
+#ifdef P_MAC_MPTHREADS
   unsigned tid = (unsigned)MPCurrentTaskID();
+#elif defined(P_VXWORKS)
+  unsigned tid = ::taskIdSelf();
+#elif defined(BE_THREADS)
+  thread_id tid = ::find_thread(NULL);
 #else
   unsigned tid = (unsigned) pthread_self();
 #endif
@@ -773,16 +957,20 @@ void PServiceProcess::PXOnAsyncSignal(int sig)
 
   strcat(msg, ", aborting.\n");
 
-  if (systemLogFile.IsEmpty()) {
-    syslog(LOG_CRIT, 
-#ifdef __BEOS__ // (Some?) BeOS versions of syslog.h have syslog() (wrongly) declared without const 
-    (char *) 
-#endif 
-    msg); 
+  if (systemLogFileName.IsEmpty()) {
+#ifdef P_VXWORKS
+  logMsg((char *)msg,0,0,0,0,0,0);
+#else
+    syslog(LOG_CRIT, msg); 
     closelog();
+#endif // !P_VXWORKS
   }
   else {
-    int fd = open(systemLogFile, O_WRONLY|O_APPEND);
+#ifdef P_VXWORKS
+    int fd = open(systemLogFileName, O_WRONLY|O_APPEND, FWRITE|FAPPEND);
+#else
+    int fd = open(systemLogFileName, O_WRONLY|O_APPEND);
+#endif // !P_VXWORKS
     if (fd >= 0) {
       write(fd, msg, strlen(msg));
       close(fd);
@@ -790,6 +978,7 @@ void PServiceProcess::PXOnAsyncSignal(int sig)
   }
 
   raise(SIGQUIT); // Dump core
+  _exit(-1); // Fail safe if raise() didn't dump core and exit
 }
 
 
@@ -817,6 +1006,7 @@ void PServiceProcess::PXOnSignal(int sig)
       }
       break;
 
+#ifdef SIGWINCH
     case SIGWINCH :
       if (currentLogLevel > PSystemLog::Fatal) {
         currentLogLevel = (PSystemLog::Level)(currentLogLevel-1);
@@ -824,6 +1014,7 @@ void PServiceProcess::PXOnSignal(int sig)
         s << "Log level decreased to " << PLevelName[currentLogLevel+1];
       }
       break;
+#endif
   }
 }
 
